function [nu1] = balk(beta, dfrq, t, index)
% [nu1]=balk(beta,dfrq,t,index)
% 
% calculates wave amplitude for a pulse from flip angle 
% duration and bandwidth
% 
% for further details see
% A. Schweiger, G. Jeschke: Principles of pulse electron paramagnetic
% resonance, Oxford University Press, 2001, p. 137
% 
% Input:
% beta    flip angle in rad
% dfrq    bandwidth in GHz
% t       pulselength in ns
% index   number of event pulse experiment
%
% Output:
% nu1     calculated wave amplitude in MHz
%
% balk checks if the conditions for fast passage are fulfilled and will
% print a warning if not.
% 
% S. Pribitzer, 2014


t=t*1e-9; % converts nanoseconds to seconds

if dfrq~= 0
    omega=abs(dfrq)*2*1e9*pi; % sweeprange in rad
    alpha=omega/t; % sweeprate in rad s^-1
    
    omega1=sqrt(3.01*alpha/(2*pi))*2/pi*beta;
    
    nu1=omega1/(2*pi)*1e-6; % converts calculated irradiation strength in MHz
    
    if alpha/omega1<0.1
        fprintf('Event %1.0f: Fast passage condition is not satisfied.\n' , index);
    elseif alpha/omega>0.1 && alpha/omega<1
        fprintf('Event %1.0f: Fast passage condition may be satisfied.\n', index);
    else
        fprintf('Event %1.0f: Fast passage condition satisfied.\n',index);
    end
    
else % calculates pulse amplitude for given flip angle for a rectangular pulse
    fprintf('Event %1.0f: Flip angle for rectangular pulse given, calculating pulse amplitude.\n',index);
    nu1=beta/(2*pi*1e6*t);
end

end

