function [gamma] = changeup(system)
% [gamma] = changeup(system)
% 
% Changeup calculates the relaxation superoperators for a given spinsystem.
% It is necessary to give the relaxation times in ns. It is possibly to
% define only one value for T1 and one for T2. This will apply the same
% relaxation time to all transitions (ZQ, SQ, DQ, ...). Relaxation times
% can be applied to different types of transitions through matrices. A
% detailed instruction can be found in the documentation.
% 
% Input:
% system
% system.spins      number of spins
% system.sqn1       spin quantum number of first spin
% system.sqn2       spin quantum number of second spin
% system.T1         double or matrix with longitudinal relaxation times, 
%                   detailed instructions how to setup the matrix can be
%                   found in the documentation.
% system.T2         double or matrix with transversal relaxation times, 
%                   detailed instructions on how to setup the matrix can be
%                   found in the documentation.
% 
% Output:
% gamma             relaxation superoperator, as used by wave_propagation
% 
% St. Pribitzer, 2014

[t11, t12]=size(system.T1);
[t21, t22]=size(system.T2);


% checks the format of the input and expands it to matrices if necessary
% for T1 times
if t11==1 && t12==1  
    
    if system.spins==1
        T1=ones(2*system.sqn1+1,2*system.sqn1+1);
        T1=T1*system.T1;
    else
        T1=ones((2*system.sqn1+1)*(2*system.sqn2+1));
        T1=T1*system.T1;
    end
    
else
    
    T1=system.T1;

end

% checks the format of the input and expands it to matrices if necessary
% for T2 times
if t21~=1 || t22~=1
    
    T2=system.T2;
    % mirrors upper triangle of matrix to lower triangle
    T2=triu(T2)+triu(T2,1)'; 
    
elseif t21==1 && t22==1 
    
    if system.spins==1
        T2=ones(2*system.sqn1+1,2*system.sqn1+1);
        T2=T2*system.T2;
    else
        T2=ones((2*system.sqn1+1)*(2*system.sqn2+1));
        T2=T2*system.T2;
    end
    
end

% Assigns default values or inf to all T1 relaxation paths which are zero.
if isfield(system,'default_T1') && ~isempty(system.default_T1)
    
    T1(T1==0)=system.default_T1;
%     fprintf('Default T1 found, assuming it for all non-defined relaxation paths.\n');
    
end

% Assigns default values or inf to all T1 relaxation paths which are zero.
if isfield(system,'default_T2') && ~isempty(system.default_T2)
    
    T2(T2==0)=system.default_T2;
%     fprintf('Default T2 found, assuming it for all non-defined relaxation paths.\n');
    
end

% creation of relaxation superoperator Gamma for a single spin
if system.spins==1
    
    nh=2*system.sqn1+1;
    gamma=zeros(nh^2);
    kk=1;
    jj=2;

    for k1=1:nh+1:nh^2       
    % calculates the positions of longitudinal relaxation, populations are 
    % never on the diagonal elements of the relaxation superoperator.
    
        for k2=k1+nh+1:nh+1:nh^2
            
            gamma(k1,k2)=-1/T1(kk,jj);
            gamma(k2,k1)=-1/T1(kk,jj);
            jj=jj+1;
            
        end
        
        kk=kk+1;
        jj=kk+1;
        
    end
    
    % populates positions of longitudinal relaxation elemts
    n=0;
    % writes T2 matrix into a vector, to allow effective processing
    T2vec=reshape(T2,nh*nh,1);
    
    for k=2:nh^2-1

            if k~=1+n*nh
               gamma(k,k)=1/T2vec(k);
            else
                n=n+1;
            end
    end
        
% creation of relaxation superoperator Gamma for a two spins 
elseif system.spins==2
    
    nh1=2*system.sqn1+1;
    nh2=2*system.sqn2+1;
    nh=nh1*nh2;
    gamma=zeros((nh1*nh2)^2);
     
    kk=1;
    jj=2;
    
    % calculates the positions of longitudinal relaxation, populations are 
    % never on the diagonal elements of the relaxation superoperator.
    for k1=1:nh+1:nh^2       
        
        for k2=k1+nh+1:nh+1:nh^2
            
            gamma(k1,k2)=-1/T1(kk,jj);
            gamma(k2,k1)=-1/T1(kk,jj);
            gamma(k1,k1)=gamma(k1,k1)+1/T1(kk,jj);
            gamma(k2,k2)=gamma(k2,k2)+1/T1(kk,jj);
            jj=jj+1;
            
        end
        
        kk=kk+1;
        jj=kk+1;
        
    end
     
    % populates positions of longitudinal relaxation elements
    n=0;
    % writes T2 matrix into a vector, to allow effective processing
    T2vec=reshape(T2,nh*nh,1);
    
    for k=2:nh^2-1

            if k~=1+n*nh
               gamma(k,k)=1/T2vec(k);
            else
                n=n+1;
            end
            
    end
    
end
    
end

