function [det_ops]=doubleplay(operators,system)
% [det_op]=doubleplay(operators, system)
% 
% doubleplay transforms a cell with detection operators into its matrix
% form. The input can consist of both strings and matrices. If the string
% is not recognized a warning is returned. Example for the input for a
% S=1/2 system:
% 
% operators{1}='Sz';
% operators{2}=[0 1; 1 0]. 
% 
% If the detection operator is already given as matrix, it is returned 
% unchanged.
% Transition/Population difference operators have the form Sz23; this 
% returns the detection operator with diagonal elements only on the second 
% and third state. 
% Sx13 is the Sx operator for the coherence between levels 1 and 3.
% For two-spin system, analagously S1z23 can be used. Keep in mind that
% Szz23 is not the same as S1z23 or S2z23. 
% To measure only one population, the strings Sz1, Szz1 or S1z1 can be
% used.
% Doubleplay checks the input for syntax and corrects it if
% necessary; eg. S1z -> Sz, S2z -> Sz for a one-spin, Sz -> S1z for a 
% two-spin system.
% 
% Input:
% operators      cell, that contains the desired detection operators as
%                string or matrices.
% system  .sops  spinoperators of the system
%         .spins number of spins in the system
%
% Output
% det_ops         cell with the detection operators in matrix form
%
% St. Pribitzer, 2014


if system.spins==1
    det_ops=cell(1,length(operators));
    for j=1:length(operators);
        if ~ischar(operators{j}) %checks if operators are string, if not, operators are already in matrix from and stored
            det_ops{j}=operators{j};
        else
            if strcmp(operators{j}(1),'-') % checks for negative sign and adapts the rest of the string. though spops can process negative detection operators, this is done at this point
                op=operators{j}(2:end);
                minus=1;
            else
                op=operators{j};
                minus=0;
            end
            b=str2double(op(2));
            if ~isnan(b);           %extracts the second element of the operator and looks for a string or a number. In the case of 
                                    % a string this tells spops if it has to build a common operator or an operator for a selected transition in the case of a double.
                ll=length(op)-2;
                newop='S';
                newop(2:1+ll)=op(3:end);
                op=newop;
            end
            if length(op)==2        % spops creates one of the common operators if possible
                if strcmp(op,'Sz')   
                    det_op=system.sops.Sz;
                elseif strcmp(op,'Sy')
                    det_op=system.sops.Sy;
                elseif strcmp(op,'Sx')
                    det_op=system.sops.Sx;
                elseif strcmp(op,'Sp')
                    det_op=system.sops.Sp;
                elseif strcmp(op,'Sm')
                    det_op=system.sops.Sm;
                else
                    error('String of detection operator %1.0f not recognized.',  j);
                end
            elseif length(op)==3    % this creates an operator for the population of a selected level, by setting the corresponding matrix element 1 and everything else to zero
                [e, ~]=size(system.sops.E);
                oo=e-str2double(op(3))+1;
                det_op=zeros(e,e);
                op1=op(1:2);
                if strcmp(op1,'Sz')   
                    det_op(oo,oo)=1;
                else
                    error('String of detection operator %1.0f not recognized.',  j);
                end
            else                    % this creates operators for selected transitions (coherences and populations)
                [e, ~]=size(system.sops.E);
                oo=e-str2double(op(3))+1;   % gets the desired transition
                oe=e-str2double(op(4))+1;
                op1=op(1:2);
                det_op=zeros(e,e);
                if strcmp(op1,'Sz')   %examines the first two elements of the string and creates the correct operator from it
                    det_op(oo,oo)=1/2;
                    det_op(oe,oe)=-1/2;
                elseif strcmp(op1,'Sx') 
                    det_op(oo,oe)=1/2;
                    det_op(oe,oo)=1/2;
                elseif strcmp(op1,'Sy')    
                    det_op(oo,oe)=-1/2*1i;
                    det_op(oe,oo)=1/2*1i;
                elseif strcmp(op1,'Sp') 
                    det_op(oo,oe)=1;
                elseif strcmp(op1,'Sm') 
                    det_op(oe,oo)=1;
                else
                    error('String of detection operator %1.0f not recognized.', j);
                end
            end
            if ~minus
                det_ops{j}=det_op;
            else
                det_ops{j}=-det_op;
            end
        end
    end
elseif system.spins==2 %string recognition in the case of two-spin system
    det_ops=cell(1,length(operators));
    for j=1:length(operators);
        if ~ischar(operators{j}) %checks if operators are string, if not, operators are already in matrix from and stored
            det_ops{j}=operators{j};
        else
            if strcmp(operators{j}(1),'-') % checks for negative sign and adapts the rest of the string. though spops can process negative detection operators, this is done at this point
                op=operators{j}(2:end);
                minus=1;
            else
                op=operators{j};
                minus=0;
            end
            a=str2double(op(2));    % extracts the second element of the operator and looks for a string or a number. In the case of 
                                    % a string this tells spops if it has to build a common operator or an operator for a selected transition in the case of a double.
                                    
            if length(op)==2        % the following routine checks the type of the string and tries to recognize what kind of operator is desired.
                if isnan(a)         % spops also tries to attempt to use "wrong" strings, eg Sx1 instead of S1x, by rearranging the string.
                    f=op(2);
                    op(2)='1';
                    op(3)=f;
                else
                    error('String of detection operator %1.0f not recognized.',  j);
                end
            elseif length(op)==3
                b=str2double(op(3));
                if isnan(a) && ~isnan(b)
                    f=op(2:3);
                    op(2)='1';
                    op(3:4)=f;
%                 elseif ~(~isnan(a) && isnan(b)) || (isnan(a) && isnan(b))
                elseif (~isnan(a) && ~isnan(b))
                    error('String of detection operator %1.0f not recognized.',  j);
                end
            elseif length(op)>=4
                b=str2double(op(3));
                c=str2double(op(4));
                if isnan(a) && ~isnan(b) && ~isnan(c)
                    f=op(2:end);
                    op(2)='1';
                    op(3:2+length(f))=f;
                elseif (~isnan(a) && ~isnan(b) && isnan(c)) || (isnan(a) && ~isnan(b) && isnan(c)) || (~isnan(a) && ~isnan(b) && ~isnan(c)) || (~isnan(a) && isnan(b) && isnan(c))
                    error('String of detection operator %1.0f not recognized.',  j);
                end
            end
            if length(op)==3 %the result of the above routine is then used to differentiate between common (case1), population (case2) and transition operators (case3)
                optype='case1'; 
            elseif length(op)==4
                optype='case2';
                [e, ~]=size(system.sops.E);
                oo=e-str2double(op(4))+1;
            else
                optype='case3';
                [e, ~]=size(system.sops.E);
                oo=e-str2double(op(4))+1;
                oe=e-str2double(op(5))+1;
                
            end
            if strncmp(op,'S1z',3)
                switch optype
                    case 'case1'
                        det_op=system.sops.S1z;
                    case 'case2'
                        det_op=zeros(e,e);
                        det_op(oo,oo)=sign(system.sops.S1z(oo,oo));
                    case 'case3'
                        det_op=zeros(e,e);
                        det_op(oo,oo)=1/2*sign(system.sops.S1z(oo,oo));
                        det_op(oe,oe)=1/2*sign(system.sops.S1z(oe,oe));
                end
            elseif strncmp(op,'S2z',3)
                switch optype
                    case 'case1'
                        det_op=system.sops.Szz;
                    case 'case2'
                        det_op=zeros(e,e);
                        det_op(oo,oo)=sign(system.sops.S1z(oo,oo));
                    case 'case3'
                        det_op=zeros(e,e);
                        det_op(oo,oo)=1/2*sign(system.sops.S1z(oo,oo));
                        det_op(oe,oe)=1/2*sign(system.sops.S1z(oe,oe));
                end
            elseif strncmp(op,'Szz',3)
                switch optype
                    case 'case1'
                        det_op=system.sops.S2z;
                    case 'case2'
                        det_op=zeros(e,e);
                        det_op(oo,oo)=sign(system.sops.S1z(oo,oo));
                    case 'case3'
                        det_op=zeros(e,e);
                        det_op(oo,oo)=1/2*sign(system.sops.S1z(oo,oo));
                        det_op(oe,oe)=1/2*sign(system.sops.S1z(oe,oe));
                end   
            elseif strncmp(op,'S1x',3)
                switch optype
                    case 'case1'
                        det_op=system.sops.S1x;
                    case 'case3'
                        det_op=zeros(e,e);
                        det_op(oo,oe)=1/2;
                        det_op(oe,oo)=1/2;
                    otherwise
                        error('String of detection operator %1.0f not recognized.',  j);
                end        
           elseif strncmp(op,'S2x',3)
                switch optype
                    case 'case1'
                        det_op=system.sops.S2x;
                    case 'case3'
                        det_op=zeros(e,e);
                        det_op(oo,oe)=1/2;
                        det_op(oe,oo)=1/2;
                    otherwise
                        error('String of detection operator %1.0f not recognized.',  j);
                end
            elseif strncmp(op,'Sxx',3)
                switch optype
                    case 'case1'
                        det_op=system.sops.Sxx;
                    case 'case3'
                        det_op=zeros(e,e);
                        det_op(oo,oe)=1/2;
                        det_op(oe,oo)=1/2;
                    otherwise
                        error('String of detection operator %1.0f not recognized.',  j);
                end
            elseif strncmp(op,'S1y',3)
                switch optype
                    case 'case1'
                        det_op=system.sops.S1y;
                    case 'case3'
                        det_op=zeros(e,e);
                        det_op(oo,oe)=-1/2*1i;
                        det_op(oe,oo)=1/2*1i;
                    otherwise
                        error('String of detection operator %1.0f not recognized.',  j);
                end   
            elseif strncmp(op,'S2y',3)
                switch optype
                    case 'case1'
                        det_op=system.sops.S2y;
                    case 'case3'
                        det_op=zeros(e,e);
                        det_op(oo,oe)=-1/2*1i;
                        det_op(oe,oo)=1/2*1i;
                    otherwise
                        error('String of detection operator %1.0f not recognized.',  j);
                end  
            elseif strncmp(op,'Syy',3)
                switch optype
                    case 'case1'
                        det_op=system.sops.Syy;
                    case 'case3'
                        det_op=zeros(e,e);
                        det_op(oo,oe)=-1/2;
                        det_op(oe,oo)=1/2;
                    otherwise
                        error('String of detection operator %1.0f not recognized.',  j);
                end
            elseif strncmp(op,'Spp',3)
                switch optype
                    case 'case1'
                        det_op=system.sops.Spp;
                    case 'case3'
                        det_op=zeros(e,e);
                        det_op(oo,oe)=1;
                    otherwise
                        error('String of detection operator %1.0f not recognized.',  j);
                end
            elseif strncmp(op,'Smm',3)
                switch optype
                    case 'case1'
                        det_op=system.sops.Smm;
                    case 'case3'
                        det_op=zeros(e,e);
                        det_op(oe,oo)=1;
                    otherwise
                        error('String of detection operator %1.0f not recognized.',  j);
                end 
            elseif strncmp(op,'S2m',3)
                det_op=system.sops.S2m;
            elseif strncmp(op,'S1m',3)
                det_op=system.sops.S1m;
            elseif strncmp(op,'S2p',3)
                det_op=system.sops.S2p;
            elseif strncmp(op,'S1p',3)
                det_op=system.sops.S1p;
            elseif strncmp(op,'Szx',3)
                det_op=system.sops.Szx;
            elseif strncmp(op,'Sxz',3)
                det_op=system.sops.Sxz;
            elseif strncmp(op,'Szy',3)
                det_op=system.sops.Sep;
            elseif strncmp(op,'Syz',3)
                det_op=system.sops.Spe;
            elseif strncmp(op,'Syx',3)
                det_op=system.sops.Syx;
            elseif strncmp(op,'Sxy',3)
                det_op=system.sops.Sxy;
            else
                error('String of detection operator %1.0f not recognized.',  j);
            end
            if ~minus
                det_ops{j}=det_op;
            else
                det_ops{j}=-det_op;
            end
        end
    end
end

end