function [wave, options] = hitbypitch(pulse, options)
% [wave, options] = hitbypitch(pulse, options)
% 
% Calculates waveform from input parameters, using mk_chirp_mixer, pickoff
% and balk. The sweeprate is linear, unless SPIDYAN is asked
% to compensate for the resonator profile. In that case either the
% resonator parameters or a custom resonator profile has to be given.
% 
% Input:
% pulse                     structure containing information on the pulse
%           pulse.frq       bandwidth, has to be vector, e.g. frq = [1.2 1.4]
%                           in GHz.
%           pulse.t         pulse length in ns, optional if nu1 and beta 
%                           are given.
%           pulse.nu1       wave amplitude in MHz, optional if t and beta 
%                           are given.
%           pulse.beta      flip angle, required only if t and nu1
%                           are both not given.
%           pulse.phase     phase of the pulse
% 
% options       structure containing additional parameters
% .resonator     structure containing the resonator settings.
%   .resonator.comp_bw  if true, SPIDYAN compensates the pulse for the
%                       resonator
%   .resonator.Ql       quality factor of the resonator.
%   .resonator.f0       resonance frequency of the resonator.
%   .resonator.s_rate   sampling rate of the resonator profile.
%   .resonator.range    optional, frequency axis [GHz] for custom resonator
%                       profiles, if not given, the resonator profile
%                       is calculated from quality Ql, f0 and s_rate.
%   .resonator.nu       optional, resonator profile of custom resonator.
%   .resonator.active   vector, containing indices of events. Resonator
%                       profile is applied to pulses found in .active, e.g.
%                       [1 3] will change the first and third event only.
% 
% .awg          structure with information on awg, optional
%               if not given default values will be applied.
%   .awg.s_rate     sampling rate [GS/s], default is 12 GS/s
%   .awg.vert_res   vertical resolution [bit], default is 10 bit
%   .awg.phase      phase shift in rad, optional, default is 0
% 
% .LO           frequency of the local oscillator [GHz], required for
%               computations in the lab frame.
% .labframe     0 or 1, default is 0. If true SPIDYAN simulates the
%               experiment in a local frame rotating at
%               options.LO+system.nu01.
% 
% Output:
% wave          structure which contains all information on the pulse
% .arbitrary    waveform as double numbers, normalized to range
%               +/-1
% .binary       waveform in integer format 0...2^vert_res - 1
% .dt           time step [ns]
% .tp           total wave time (multiple of .dt) [ns]
% .nu1          Pulse amplitude in MHz.
% .beta         Flip angle in rad.
% .Q            Adiabaticity factor of pulse.
% .frq          Initial and final frequency of sweep.
% .t_rise       Rise time of the pulse in ns.
% .phase        phase of the pulse
% .vert_res     Vertical resolution of AWG.
% 
% S. Pribitzer, 2014

dfrq=pulse.frq(2)-pulse.frq(1);

% checks for sufficient pulse parameters (pulse length, irradtiation 
% strength and/or flip angle). missing parameters are calculated by calling
% pickoff or balk if possible.
if ~isempty(pulse.tp) && isempty(pulse.nu1) &&  ~isempty(pulse.Q) && isempty(pulse.beta)

    if dfrq~=0
        pulse.nu1=1000*sqrt(pulse.Q*dfrq/(2*pi*pulse.tp));
    else
        error('Error while setting up event %1.0f: Can not create pulse from adiabaticity factor for a hard pulse.', pulse.eventno);
    end
           
elseif isempty(pulse.tp) && ~isempty(pulse.nu1) 
    
    % calculates pulse length
    if ~isempty(beta)                       
        pulse.tp=pickoff(pulse.beta,dfrq,pulse.nu1,pulse.eventno);
    else
        error('Error while setting up event %1.0f: Only pulselength given.', pulse.eventno);
    end
    
elseif ~isempty(pulse.tp) && isempty(pulse.nu1)
    
    % calculates pulse strength
    if ~isempty(pulse.beta)
        pulse.nu1=balk(pulse.beta,dfrq,pulse.tp, pulse.eventno);
    else
        error('Error while setting up event %1.0f: Only irradtion strength given.', pulse.eventno);
    end
    
elseif isempty(pulse.tp) && isempty(pulse.nu1)
    
     error('Error while setting up event %1.0f: Not enough chirp information given.', pulse.eventno);
end

if dfrq~=0
    
    % calculates adiabaticity factor 
    pulse.Q=2*pi*(pulse.nu1/1000)^2*pulse.tp/dfrq;
    
end

% stores parameters in structure chirp for mk_chirp_mixer
chirp.nu_init=pulse.frq(1);
chirp.nu_final=pulse.frq(2);
chirp.t_rise=pulse.t_rise;
chirp.tp=pulse.tp;
chirp.phase=pulse.phase;

%the pulseform is calculated by calling mk_chirp_mixer with the pulse parameters
[wave, options]=mk_chirp_mixer(chirp,options); 

% other fields that are added to the structure of the pulse
wave.nu1=pulse.nu1; 
wave.beta=pulse.beta;
wave.tp=pulse.tp;
wave.Q=pulse.Q;
wave.frq=pulse.frq;
wave.t_rise=chirp.t_rise;
wave.phase=pulse.phase;

end

