function [state, detected_signal, experiment, options, tables] = homerun(state, system, experiment, options, tables)
% [state, detected_signal, experiment, options, tables] = homerun(state, system, experiment, options, tables)
% 
% homerun propagates the state of any spinsystem throughout an experiment,
% by calling wave_propagation. It is possible to chose between Hilbert and 
% Liouville space and to switch relaxation on or off. If only the final 
% state of the system is of interest it is possible to input not detection 
% operators. This will propagate directly to the final state and return
% empty cells for the expectation values.
% 
% Input/Output:
% state         state of the system, has to be in matrix form, can be 
%               generated with setup.
% 
% system        structure with the spin system
% .nu01         resonance frequency of spin 1 in GHz, optional only if
%               system.ham is given
% .nu02         resonance frequency of spin 2 in GHz, optional
% .ham          hamilton operator the system, can be given or
%               is created by setup.m
% .sqn1         nuclear spin of the first spin, is assumed as 1/2 if no 
%               value is given
% .sqn2         nuclear spin of the second spin, optional
% .init_state   initial state, can be a magnetization vector, density matrix or
%               empty. if empty thermal equilibrium will be assumed. Example 
%               for a vector system.init_state=[0 0 1].
% .spins        number of spin, optional, if 2 and no sqn2 (empty or not
%               defined), sqn2 is assumed to be 1/2.
% .A,.B         coupling constants, only needed for a two spin system
% .D            symmetric zero-field-splitting parameter in GHz
% .E            asymmetric zero-field-splitting parameter in GHz
% .theta        first angle in rad, for calculation of ZFS hamiltonian
% .phi          second angle in rad, for calculation of ZFS hamiltonian
% .omega1       zero-field-splitting frequency, calculated from system.D 
%               with fastball.
% .omega21,     zero-field-splitting frequencies for higher order
% .omega22      Hamiltonian terms                        
% .T1           double or matrix with longitudinal relaxation times, 
%               detailed instructions see below.
% .T2           vector with transversal relaxation times, detailed
%               instructions see below.
% .gamma        relaxation matrix, optional, else calculated from T1 and T2
% .default_T1   a default value for T1 relaxation, optional. Is applied to 
%               all non-defined T1 pathways.
% .default_T2   a default value for T2 relaxation, optional. Is applied to 
%               all non-defined T2 pathways
% .eq           equilibrium state, density matrix, optional, created as
%               thermal equilibrium if not else specified
% .R            relaxation matrix in Hilbert space, only available for
%               system with one spin and S=1/2
% .sops         structure containing the spin operators
% 
% experiment
% .pulse      structure which contains all information on the
%            pulses, empty element for free evolution.
%   .pulse{k}   structure with the k-th pulse
%   .pulse{k}.tp          total wave time (multiple of .dt) [ns]
%   .pulse{k}.dt          time step [ns]
%   .pulse{k}.t           time axis of wave
%   .pulse{k}.vert_res    vertical resulotion of wave.binary
%   .pulse{k}.arbitrary   waveform as double numbers, normalized to range 
%                         +/-1
%   .pulse{k}.binary      waveform in integer format 0...2^vert_res - 1
%   .pulse{k}.nu_init     Initial frequency of the sweep, if calculations
%                         are in the labframe, this will be frq(1)+LO
%   .pulse{k}.nu_final    Final frequency of the sweep, if calculations
%                         are in the labframe, this will be frq(2)+LO
%   .pulse{k}.nu1         pulse amplitude in MHz
%   .pulse{k}.beta        flipangle of pulse
%   .pulse{k}.frq         vector, excitation band
%   .pulse{k}.t_rise      rise time of the pulse
%   .pulse{k}.pcycle      phase cycling of the pulse, as defined in
%                         sequence.pcycle, 0 if no phase cycling
% .tp       vector containing the length of the pulses,
%           interpulse delays and detection time.
% .dt       time resolution (timesteps) of the experiment
% .i        index, used by homerun to process the experiment
% .o        offset, is used to calculate the time axis, will be
%           updated by homerun.
% .t        structure, contains the individual time axis of every event
% .taxis    vector, which contains time axis of the entire experiment
% 
% .options      structure containing additional parameters
% .det_op       cell containing the detection operators as string (S1z,S2z,
%               Sz,...) or in matrix matrix form. For more details see
%               documentation, can be empty.
% .detect       cell containing the detection operators in matrix form, if
%               input det was empty or not given, detect is empty.
% .no_detection cell, containing the event, that to exclude from detection,
%               eg.:
%                       options.no_detection={1, 3}
%               No detection during the 1st and 3rd event of the
%               experiment.
% .relaxation   0 or 1. if true, calculation includes relaxation. Uses
%               Liouville space. default is 0.
% .propagator   0 or 1. if true, the system is propagated with Hilbert space
%               with a propagator. Can not be true for systems with more 
%               than two levels AND relaxation on.
% .LO           frequency of the local oscillator [GHz], required for
%               computations in the lab frame.
% .labframe     0 or 1, default is 0. If true SPIDYAN simulates the
%               experiment in a local frame rotating at
%               options.LO+system.nu01.
% .down_conversion    0 or 1, if true, the detected signal will be 
%               transformed into the lab frame. Default is on.
% .no_dc        cell which containing the signals not to be down converted as
%               string or index of detection operator, optional.
% .cutoff_freq  cutoff frequency of the applied filter for down conversion
%               in MHz. Usually 200 MHz is a good value to start with. 
% .ui           0 or 1, optional, default is false. A simple
%               user-interface, which can be used to input relaxation
%               times.
% .truncation   if true, SPIDYAN checks for transitions outside the Nyquist
%               band and sets corresponding elements in the excitation
%               operator to zero. Only possible for single spin system.
% .awg          structure with information on awg, optional
%               if not given default values will be applied.
%   .awg.s_rate     sampling rate [GS/s], default is 12 GS/s
%   .awg.vert_res   vertical resolution [bit], default is 10 bit
%   .awg.phase      phase shift in rad, optional, default is 0
% 
% .display_filter   0 or 1, optional, default is false. If true displays the
%                   frequency response of the filter used for downconversion.
% .filtertruncation optional, strength of the filter for function 
%                   resonator, default is 1000. 
% .direct_evolution     vector, contains the indices of events which to
%                       propagate directly. can be used for interpulse
%                       delays to speed up calculation.
% .complex_excitation   0 or 1, optional, default is false. If true, complex
%                       microwave pulses are computed, which can be used for a
%                       complex excitation.
% .resonator     structure containing the resonator settings.
%   .resonator.comp_bw  if true, SPIDYAN compensates the pulse for the
%                       resonator
%   .resonator.Ql       quality factor of the resonator.
%   .resonator.f0       resonance frequency of the resonator.
%   .resonator.s_rate   sampling rate of the resonator profile.
%   .resonator.range    optional, frequency axis [GHz] for custom resonator
%                       profiles, if not given, the resonator profile
%                       is calculated from quality Ql, f0 and s_rate.
%   .resonator.nu       optional, resonator profile of custom resonator.
%   .resonator.active   vector, containing indices of events. Resonator
%                       profile is applied to pulses found in .active, e.g.
%                       [1 3] will change the first and third event only.
% 
% 
% tables        structure containing lookup tables
% .sigss        optional, table with steadystate density operators. if 
%               available, accelerates calulcation a lot. can only be used,
%               for pulses with the same nu1.
% .L            optional, table with liouvillians.
%               if given, will accelerate calulcation a lot. 
%               can only be used, if the exact same pulse was 
%               already used on the same system in a previous 
%               experiment.
% .nu1          irradtiation strength of the pulse for which
%               the tables are given. If not the same as the
%               applied pulse, tables will be calculated new.
% 
% 
% detected_signal
% .sf   n-dimensional vector, containing the time traces in the simulation frame,
%       n equals the number of detection operators.
% .dc   n-dimensional vector with the entire time trace of the down converted
%       signal, only available if options.down_conversion is true
% .t    vector, the time axis of the entire experiment.
% .signals    structure which contains the individual signals of each event
% .signals.sf cell, with the individual signals of each event, in simulation frame.
% .signals.dc  cell, containing the individual signals of each event,
%             down converted, only available if down_conversion is true.
% .signals.t  cell, time axis of the individual events.
% 
% 
% St. Pribitzer, 2014


% Check options and if not available assumes default options: 
if ~isfield(options,'down_conversion') || isempty(options.down_conversion)
    options.down_conversion=1;
end


% creates the cells for the detected signals, if detection is requested
if isempty(options.detect)
    detected_signal=[];
elseif isfield(options,'no_detection') && ~isempty(options.no_detection)
    s=length(experiment.tp)-length(options.no_detection);
    signal=cell(1,s);
    signalf=cell(1,s);
    times=cell(1,s);
else
    signal=cell(1,length(experiment.tp));
    signalf=cell(1,length(experiment.tp));
    times=cell(1,length(experiment.tp));
    options.no_detection=[];
end

tablesnew=cell(1,length(experiment.tp));

% indices used for the loop
c=0;
r=1;

% due to the choice of the time grid, the initial state has to be rotated
% back by half a timestep
if system.spins==2
    backrot = expm(1i*system.sops.S1z*2*pi*system.nu01*experiment.dt/2+1i*system.sops.S1z*2*pi*system.nu02*experiment.dt/2);
    state = backrot*state*backrot';
elseif system.spins==1
    backrot = expm(1i*system.sops.Sz*2*pi*system.nu01*experiment.dt/2);
    state = backrot*state*backrot';
end

for k=1:length(experiment.tp)
    if ~exist('tables','var') || isempty(tables)    % check if tables are available. this can speed up the calculation, but can only be used for the same pulse. 
                                                    % If tables are given, wave_propagation will check again if it is the correct table.
        if ~isempty(options.detect)  && ~isempty(options.no_detection) % propagate directly to state without detection
            
            if ismember(k,options.no_detection)
                                
                [state, ~, experiment, table] = wave_propagation(state, system, experiment, options, []);
               
            else % propagates with detection
                
                [state, signal{r}, experiment, table] = wave_propagation(state, system, experiment, options, []);
                [~, x]= size(signal{r});    % check if signal length is sufficient for down_conversion  
                
                if options.down_conversion && x>10
                    [signalf{r},options.filter]=strike(signal{r}, experiment.t{k}, system.nu01, options);
                else
                    signalf{r}=[];
                end
                
                c=c+x;
                times{r}=experiment.t{k};
                r=r+1;

            end 
            
        elseif ~isempty(options.detect) && isempty(options.no_detection)       % propagate to final state with detection
            
            [state, signal{r}, experiment, table] = wave_propagation(state, system, experiment, options, []);
            [~, x]= size(signal{r});     % check if signal length is sufficient for down_conversion    
            
            if options.down_conversion && x>10
                [signalf{r},options.filter]=strike(signal{r}, experiment.t{k}, system.nu01, options);
            else
                signalf{r}=[];
            end
            
            c=c+x;
            times{k}=experiment.t{r};
            r=r+1;
            
        else
            [state, ~, experiment, table] = wave_propagation(state, system, experiment, options, []);
        end
        
        tablesnew{k}=table;
        
        if ~isempty(tablesnew{k})
            tablesnew{k}.ham=system.ham;
        end
        
    else % runs_wave propagation with tables available
        
        if k<=length(tables)
            table=tables{k};
        else
            table=[]; % if tables are not given for the current element, wave_propagation is called with an empty structure, which will cause it to recalculate the tables
        end
        
        if ~isempty(options.detect)  && ~isempty(options.no_detection)
            
            if ismember(k,options.no_detection) % propagate directly to state without detection
                
                [state, ~, experiment, tablenew] = wave_propagation(state, system, experiment, options, table);
                
            else
            
                [state, signal{r}, experiment, tablenew] = wave_propagation(state, system, experiment, options, table);
                [~, x]= size(signal{r});     % check if signal length is sufficient for down_conversion     
                
                if options.down_conversion && x>10
                    [signalf{r},options.filter]=strike(signal{r}, experiment.t{k}, system.nu01, options);
                else
                    signalf{r}=[];
                end
                
                c=c+x;
                times{r}=experiment.t{k};
                r=r+1;

            end   
            
        elseif ~isempty(options.detect) && isempty(options.no_detection)        % propagate to final state with detection for all events
            
            [state, signal{r}, experiment, tablenew] = wave_propagation(state, system, experiment, options, table);
            [~, x]= size(signal{r});     % check if signal length is sufficient for down_conversion  
            
            if options.down_conversion && x>10
                [signalf{r},options.filter]=strike(signal{r}, experiment.t{k}, system.nu01, options);
            else
                signalf{r}=[];
            end
            
            c=c+x;
            times{k}=experiment.t{r};
            r=r+1;
            
        else
            [state, ~, experiment, tablenew] = wave_propagation(state, system, experiment, options, table);
        end
        
        if ~isempty(table) || ~isempty(tablenew)
            tablenew.ham=system.ham;
        end
        
        if ~isequal(tablenew,table)
            tables{k}=tablenew;
        end
        
   end
end

if ~isempty(options.detect) % this rearranges the detected signals and time axis for the output
    
    detected_signal.sf=zeros(length(options.detect),c-experiment.i-2);
    experiment.taxis=zeros(1,c-experiment.i-2);
    detected_signal.sf(:,1)=signal{1}(:,1);
    experiment.taxis(1,1)=times{1}(1);
    
    a=2;
    
    for k=1:length(signal)
        
        [~,y]= size(signal{k});
        b=a+y-2;
        detected_signal.sf(:,a:b)=signal{k}(:,2:end);
        experiment.taxis(1,a:b)=times{k}(2:end);
        a=b+1;
        
    end
    
    [~, x]= size(detected_signal.sf);     % check if signal length is sufficient for down_conversion           
    
    if options.down_conversion && x>10
        [detected_signal.dc,options.filter]=strike(detected_signal.sf, experiment.taxis, system.nu01, options);
        signals.dc=signalf;
    else
        detected_signal.dc=[];
        signals.dc=[];
    end
    
    signals.sf=signal;
    detected_signal.signals=signals;
    detected_signal.t=experiment.taxis;
    detected_signal.signals.t=times;
    
else
    
    detected_signal=[];
    
end

if ~isempty(tablesnew{1}) % If new tables have been created, those are returned
    tables=tablesnew;
end

end

        

