function [wave, options]=mk_chirp_mixer(chirp,options)
% [wave, options]=mk_chirp_mixer(chirp,options)
%
% generates a chirp waveform for spin dynamics simulation or transfer to an
% arbitrary waveform generator (AWG). The sweeprate is linear, unless SPIDYAN
% is supposed to compensate the resonator profile. In that case either the
% resonator parameters or a custom resonator profile has to be given.
%
% Input:
% chirp                     chirp parameters
% chirp.nu_init             initial frequency
% chirp.nu_final            final frequency
% chirp.tp                  pulse length
% chirp.t_rise              rise time
% chirp.phase               initial phase [rad], defaults to 0
% 
% options       structure containing additional parameters
% .resonator    structure containing the resonator settings.
%   .resonator.comp_bw  if true, SPIDYAN compensates the pulse for the
%                       resonator
%   .resonator.Ql       quality factor of the resonator.
%   .resonator.f0       resonance frequency of the resonator.
%   .resonator.s_rate   sampling rate of the resonator profile.
%   .resonator.range    optional, frequency axis [GHz] for custom resonator
%                       profiles, if not given, the resonator profile
%                       is calculated from quality Ql, f0 and s_rate.
%   .resonator.nu       optional, resonator profile of custom resonator.
%   .resonator.active   vector, containing indices of events. Resonator
%                       profile is applied to pulses found in .active, e.g.
%                       [1 3] will change the first and third event only.
% 
% .awg          structure with information on awg, optional
%               if not given default values will be applied.
%   .awg.s_rate     sampling rate [GS/s], default is 12 GS/s
%   .awg.vert_res   vertical resolution [bit], default is 10 bit
%   .awg.channels   1 for only real output, 2 for real/imaginary
%                   output, default is to 1
% 
% .LO           frequency of the local oscillator [GHz], required for
%               computations in the lab frame.
% .labframe     0 or 1, default is 0. If true SPIDYAN simulates the
%               experiment in a local frame rotating at
%               options.LO+system.nu01.
% 
% 
% Output:
% wave                      structure which contains all information on the
%                           waveform.
% wave.arbitrary            waveform as double numbers, normalized to range
%                           +/-1
% wave.binary               waveform in integer format 0...2^vert_res - 1
% wave.dt                   time step [ns]
% wave.tp                   total wave time (multiple of .dt) [ns]
% 
% no plausibility checks of parameters are implemented
% 
% Stephan Pribitzer, 2014

%creates additional chirp parameters


if ~isfield(chirp,'scale'),
    chirp.scale=1;
end

awg=options.awg;

% vertical resolution in steps
res=2^awg.vert_res; 

% converts the excitation band to the labframe if requested
if isfield(options,'labframe') && ~isempty(options.labframe) && options.labframe && isfield(options,'LO') && ~isempty(options.LO)
    
    chirp.nu_final=chirp.nu_final+options.LO;
    chirp.nu_init=chirp.nu_init+options.LO;
    
end

% creates higher sampling rate if given sampling rates is not sufficient
if awg.s_rate<2*chirp.nu_final || awg.s_rate<2*chirp.nu_init
    
    dt=1/(4*(chirp.nu_final));
    warning('Sampling rate of AWG not sufficient for requested pulse. Assuming sampling rate of %3.1f GS/s.',1/dt);
    options.awg.s_rate=1/dt;
    
else
    
    dt=1/awg.s_rate;
    
end

if chirp.t_rise==chirp.tp
    chirp.t_rise=chirp.tp-chirp.tp/10;
    warning('Rise time of chirp is the same as pulse length. New rise time is %3.1f ns.',chirp.t_rise);
elseif chirp.t_rise>chirp.tp
    chirp.t_rise=chirp.tp-chirp.tp/10;
    warning('Rise time of chirp is larger than pulse length. New rise time is %3.1f ns.',chirp.t_rise);
end

% creates time axis
n=ceil(chirp.tp/dt);
wave.tp=n*dt;
wave.dt=dt;
t=0:wave.dt:wave.tp-wave.dt;
wave.t=t;

if isfield(options,'resonator') && isfield(options.resonator,'comp_bw') && ~isempty(options.resonator.comp_bw) && options.resonator.comp_bw && isfield(options.resonator,'comp_bw') && options.resonator.active

    %check for completness
    if ~isfield(options.resonator,'s_rate') || isempty(options.resonator.s_rate)
        options.resonator.s_rate=36*3;       
    end
    
    if isfield(options.resonator,'nu1') && isfield(options.resonator,'range')
          resntor.range=options.resonator.range;
          resntor.nu1=options.resonator.nu1;
          
    elseif isfield(options.resonator,'Ql') && isfield(options.resonator,'f0')    
        npts=2^14;
        f = transpose((-npts/2:npts/2-1)/npts*options.resonator.s_rate);
        Hid=1./(1+1i*options.resonator.Ql*(f/options.resonator.f0-options.resonator.f0./f));
        if ~isfield(options,'labframe') || ~options.labframe
            resntor.range=f-options.LO;
        else
            resntor.range=f;           
        end
        resntor.nu1=Hid;
        wave.resonator = resntor;
    else
        warning('Resonator compensation requested, but resonator not defined, assuming linear chirp.');
        resntor.range = [0 20];
        resntor.nu1 = [1 1];
    end
    
    % check the resonator ranges
    if chirp.nu_init<min(resntor.range),
        warning('Requested min. frq. of %4.2f GHz is less than min. frq. of %4.2f GHz of the known resonator profile.',chirp.nu_init,min(resntor.range));
    end
    if chirp.nu_final>max(resntor.range),
        warning('Requested max. frq. of %4.2f GHz is more than max. frq. of %4.2f GHz of the known resonator profile.',chirp.nu_final,max(resntor.range));
    end


    % include the resonator nu1 via the time per frequency: 
    % dt/df = 1/(mean_alpha*nu1^2)
    % frange=linspace(chirp.nu_init,chirp.nu_final,length(t));
    frange=chirp.nu_init:sign(chirp.nu_final-chirp.nu_init)*0.001:chirp.nu_final;
    if length(frange) < 2
        frange = ones(2,1)*chirp.nu_init;
    end
    sprofile=interp1(resntor.range,abs(resntor.nu1),frange,'pchip');
    mean_alpha=trapz(1./sprofile.^2)/t(end); % 2pi/Qref
    dt_df = 1./sprofile.^2/mean_alpha;
    wave.qref = 2*pi/mean_alpha/0.001; % division by 0.001 (trapz step size)

    % check for resonator nu1 in MHz
    if max(abs(resntor.nu1)) > 1
        wave.qref = wave.qref * 0.001^2; % make MHz to GHz 
    end
    % get time tau vs freq from dt/df
    tau = cumtrapz(dt_df);

    % reparametrize tau(nu) to nu(tau)
    nu = interp1(tau,frange,t,'pchip');
    wave.nu = nu;

    % calculate the wave
    real_wave=chirp.scale.*cos(2*pi*cumtrapz(nu)*wave.dt+chirp.phase);
    imag_wave=chirp.scale.*sin(2*pi*cumtrapz(nu)*wave.dt+chirp.phase);
    
else

    % compute chirp
    alpha=(chirp.nu_final-chirp.nu_init)/chirp.tp;
    real_wave=cos(2*pi*(chirp.nu_init*t+(alpha/2)*t.^2)+chirp.phase);
    imag_wave=sin(2*pi*(chirp.nu_init*t+(alpha/2)*t.^2)+chirp.phase);

end

% round edges, if requested
nr=round(chirp.t_rise/wave.dt);
if nr>0,
    dr=nr*wave.dt;
    tr=dr:-wave.dt:0;
    edge=cos(pi*tr/(2*dr));
    real_wave(1:nr+1)=edge.*real_wave(1:nr+1);
    imag_wave(1:nr+1)=edge.*imag_wave(1:nr+1);
    edge=fliplr(edge);
    real_wave(end-nr:end)=edge.*real_wave(end-nr:end);
    imag_wave(end-nr:end)=edge.*imag_wave(end-nr:end);    
end

% compute integer input values for AWG
real_binary=floor(res*(real_wave+1)/2);
real_binary(real_binary==res)=res-1;

imag_binary=floor(res*(imag_wave+1)/2);
imag_binary(imag_binary==res)=res-1;

if awg.channels==2,
    wave.binary=real_binary+1i*imag_binary;
else
    wave.binary=real_binary;
end

% compute normalized with with vertical resolution of AWG
real_wave=2*(real_binary/res-0.5);
imag_wave=2*(imag_binary/res-0.5);

if awg.channels==2,
    wave.arbitrary=real_wave+1i*imag_wave;
else
    wave.arbitrary=real_wave;
end

wave.vert_res = awg.vert_res;
   
end