function [system, state, options]=setup(system, options)
%  [system, state, options]=setup(system, options)
% 
%  setup processes the input parameters of system, so that they can be
%  used to call the function homerun. If the input is incomplete a
%  warning will be printed.
%  Following Hamiltonians are supported:
% 
%  Hamiltonian for a single spin with I=1/2
%           H=wSz
% 
%  Hamiltonian for a single spin with I>1/2
%           H=wSz+DSz^2+(D2-D1)Sz^2
%  
%  Hamiltonian for two coupled spins
%           H=wIz+wSz+ASzIz+BSxIx
% 
% Input/Output:
% system        structure with the spin system
% .nu01         resonance frequency of spin 1 in GHz, optional only if
%               system.ham is given
% .nu02         resonance frequency of spin 2 in GHz, optional
% .ham          hamilton operator the system, can be given or
%               is created by setup.m
% .sqn1         nuclear spin of the first spin, is assumed as 1/2 if no 
%               value is given
% .sqn2         nuclear spin of the second spin, optional
% .init_state   initial state, can be a magnetization vector, density matrix or
%               empty. if empty high temperature limit will be assumed. Example 
%               for a vector system.init_state=[0 0 1].
% .spins        number of spin, optional, if 2 and no sqn2 (empty or not
%               defined), sqn2 is assumed to be 1/2.
% .A,.B         coupling constants, only needed for a two spin system
% .D            symmetric zero-field-splitting parameter in GHz
% .E            asymmetric zero-field-splitting parameter in GHz
% .theta        first angle in rad, for calculation of ZFS hamiltonian
% .phi          second angle in rad, for calculation of ZFS hamiltonian
% .omega1       zero-field-splitting frequency, calculated from system.D 
%               with fastball.
% .omega21,     zero-field-splitting frequencies for higher order
% .omega22      Hamiltonian terms                        
% .T1           double or matrix with longitudinal relaxation times, 
%               detailed instructions see below.
% .T2           vector with transversal relaxation times, detailed
%               instructions see below.
% .gamma        relaxation matrix, optional, else calculated from T1 and T2
% .default_T1   a default value for T1 relaxation, optional. Is applied to 
%               all non-defined T1 pathways.
% .default_T2   a default value for T2 relaxation, optional. Is applied to 
%               all non-defined T2 pathways
% .eq           equilibrium state, density matrix, optional, created as
%               high temperature limit if not else specified
% .R            relaxation matrix in Hilbert space, only available for
%               system with one spin and S=1/2
% .sops         structure containing the spin operators
%
% 
% .options      structure containing additional parameters
% .det_op       cell containing the detection operators as string (S1z,S2z,
%               Sz,...) or in matrix matrix form. For more details see
%               documentation, can be empty.
% .detect       cell containing the detection operators in matrix form, if
%               input det was empty or not given, detect is empty.
% .no_detection cell, containing the event, that to exclude from detection,
%               eg.:
%                       options.no_detection={1, 3}
%               No detection during the 1st and 3rd event of the
%               experiment.
% .relaxation   0 or 1. if true, calculation includes relaxation. Uses
%               Liouville space. default is 0.
% .propagator   0 or 1. if true, the system is propagated with Hilbert 
%               space with a propagator. Can not be true for systems with 
%               more than two levels AND relaxation on.
% .LO           frequency of the local oscillator [GHz], required for
%               computations in the lab frame.
% .labframe     0 or 1, default is 0. If true SPIDYAN simulates the
%               experiment in a local frame rotating at
%               options.LO+system.nu01.
% .down_conversion    0 or 1, if true, the detected signal will be 
%               transformed into the lab frame. Default is on.
% .no_dc      cell which containing the signals not to be filtered as
%               string or index of detection operator, optional.
% .cutoff_freq  cutoff frequency of the applied filter for down conversion
%               in MHz. Usually 100 MHz is a good value to start with. 
% .ui           0 or 1, optional, default is false. A simple
%               user-interface, which can be used to input relaxation
%               times.
% .truncation   if true, SPIDYAN checks for transitions outside the Nyquist
%               band and sets corresponding elements in the excitation
%               operator to zero. Only possible for single spin system.
% .awg          structure with information on awg, optional
%               if not given default values will be applied.
%   .awg.s_rate     sampling rate [GS/s], default is 12 GS/s
%   .awg.vert_res   vertical resolution [bit], default is 10 bit
%   .awg.channels   1 for only real output, 2 for real/imaginary
%                   output, default is to 1
%   .awg.phase      phase shift in rad, optional, default is 0
%
% .display_filter   0 or 1, optional, default is false. If true displays 
%                   the frequency response of the filter used for 
%                   downconversion.
% .filtertruncation optional, strength of the filter for function 
%                   resonator default is 1000. 
% .direct_evolution     vector, contains the indices of events which to
%                       propagate directly. can be used for interpulse
%                       delays to speed up calculation.
% .complex_excitation   0 or 1, optional, default is false. If true, 
%                       complex microwave pulses are computed, which can be
%                       used for a complex excitation.
% .resonator     structure containing the resonator settings.
%   .resonator.comp_bw  if true, SPIDYAN compensates the pulse for the
%                       resonator
%   .resonator.Ql        quality factor of the resonator.
%   .resonator.f0        resonance frequency of the resonator.
%   .resonator.s_rate    sampling rate of the resonator profile.
%   .resonator.range    optional, frequency axis [GHz] for custom resonator
%                       profiles, if not given, the resonator profile
%                       is calculated from quality Ql, f0 and s_rate.
%   .resonator.nu       optional, resonator profile of custom resonator.
%   .resonator.active   vector, containing indices of events. Resonator
%                       profile is applied to pulses found in .active, e.g.
%                       [1 3] will change the first and third event only.
% 
% state     spin density matrix, calculated from initial state or high temperature limit
%
% St. Pribitzer, 2014
 
default_A=0; % [GHz]
default_B=0; % [GHz]
default_T1=100000;
default_T2=100000;

if isfield(system,'ns1') && ~isfield(system,'sqn1')
    warning('Setup called with system.ns1. Please use system.sqn1 instead.');
    system.sqn1=system.ns1;
elseif ~isfield(system,'sqn1') || isempty(system.sqn1)     % sets first spin to 1/2 if not given
    system.sqn1=1/2;
    warning('Setup.m called without spin quantum number, assuming 1/2.');
end

if isfield(system,'ns2')
    system.sqn2=system.ns2;
    warning('Setup called with system.ns2. Please use system.sqn2 instead.');
end

if ~isfield(system,'spins') % determines number of spins, and creates a second spin if needed
   
    if ~isfield(system,'sqn2') && ~isfield(system,'nu02')
        system.spins=1;
    elseif isfield(system,'nu02') && (~isfield(system,'sqn2') || isempty(system.sqn2))
        system.sqn2=1/2;
        system.spins=2;
        warning('Setup.m called with a resonance frequency for second spin, but no spin quantum number, assuming 1/2.');
    else
        system.spins=2;
    end
    
end

if sign(system.sqn1)<0
    system.sqn1=abs(system.sqn1);
    warning('Negativ spin quantum number for first spin. Using %1.1f instead.',system.sqn1);
end

if system.spins==2 && sign(system.sqn2)<0
    system.sqn2=abs(system.sqn2);
    warning('Negativ spin quantum number for second spin. Using %1.1f instead.',system.sqn2);
end

if isfield(options,'relaxation') && options.relaxation &&  isfield(options,'ui') && ~isempty(options.ui) && options.ui
    [system] = ui_relaxation(system, options); % starts user interface if requested
end

if ~isfield(system,'default_T1') || isempty(system.default_T1)
    system.default_T1=default_T1;
end
if ~isfield(system,'default_T2') || isempty(system.default_T2)
    system.default_T2=default_T2;
end

if isfield(system,'T1') && ~isempty(system.T1) && ((~isfield(system,'T2') || isempty(system.T2)))
    % if only T1 relaxation times are given, T2 times are set to default
    % values of inf.
    if system.spins==2
        nh=(system.sqn1*2+1)*(system.sqn2*2+1);
    elseif system.spins==1
        nh=(system.sqn1*2+1);
    end
    
    system.T2 =  ones(nh,nh);
    system.T2 = system.T2* system.default_T2;
    warning('Only T1 relaxation times found, assuming default T2 = %5.2f ns.', system.default_T2)
    
elseif isfield(system,'T2') && ~isempty(system.T2) && ((~isfield(system,'T1') || isempty(system.T1)))
    % if only T2 relaxation times are given, T1 times are set to default
    % values of inf.
    if system.spins==2
        nh=(system.sqn1*2+1)*(system.sqn2*2+1);
    elseif system.spins==1
        nh=(system.sqn1*2+1);
    end
    
    system.T1 =  ones(nh,nh);
    system.T1 = system.T1* system.default_T1;
    warning('Only T2 relaxation times found, assuming default T1 = %5.2f ns.', system.default_T1);
    
end

if ~isfield(system,'sops') || isempty(system.sops) % creates spin operators
    system.sops=hitandrun(system);
end

% if no Hamiltonian is given, this checks if the input is sufficient to
% calculate one
if system.spins==2 && ~isfield(system,'ham') && ~isfield(system,'nu01')
    error('Resonance frequency of the first spin is missing.');
elseif system.spins==2 && ~isfield(system,'ham') && ~isfield(system,'nu02')
    error('Resonance frequency of the second spin is missing.');
elseif system.spins==1 && ~isfield(system,'ham') && ~isfield(system,'nu01')
    error('No resonance frequency given.');
end

if isfield(system,'ham') % if no Hamiltonian is available, its created here  
elseif system.spins==1
    % if calculations are requested in the labframe, resonance frequencies
    % are upconverted
    if isfield(options,'labframe') && ~isempty(options.labframe) && options.labframe && isfield(options,'LO') && ~isempty(options.LO)
        system.ham=2*pi*(system.nu01+options.LO)*system.sops.Sz;
    else
        system.ham=2*pi*system.nu01*system.sops.Sz;
    end
    % creates ZFS hamiltonian
    if system.sqn1~=1/2
        if isfield(system,'D') && isfield(system,'E') && isfield(system,'theta') && isfield(system,'phi')
            if isfield(options,'LO') && ~isempty(options.LO)
                hamZ=(system.nu01+options.LO)*system.sops.Sz;
            else
                hamZ=(system.nu01)*system.sops.Sz;
            end
            
            hamtot=ZFS(system.sqn1,system.D,system.E,system.theta,system.phi,hamZ);
            
            if isfield(options,'labframe') && ~isempty(options.labframe) && ~options.labframe && isfield(options,'LO') && ~isempty(options.LO)
                hamLO=2*pi*(options.LO)*system.sops.Sz;
                system.ham=2*pi*hamtot-hamLO;
            else
                system.ham=2*pi*hamtot;
            end
        else
            fprintf('Setup called without ZFS for S>1/2, assuming zero.\n');
        end
    end
    
elseif system.spins==2
    % for calculations in labframe, SPIDYAN checks if one of the two
    % resonance frequencies is a nuclear spin (if nu0 < 100 MHz). Such
    % spins are excluded from upconversion.
    if isfield(options,'labframe') && ~isempty(options.labframe) && options.labframe && isfield(options,'LO') && ~isempty(options.LO)
        if system.nu01<0.1
            nu01=system.nu01;
        else
            nu01=system.nu01+options.LO;
        end
        if system.nu02<0.1
            nu02=system.nu02;
        else
            nu02=system.nu02+options.LO;
        end
        system.ham=2*pi*nu01*system.sops.S1z+2*pi*nu02*system.sops.S2z;
    else
        system.ham=2*pi*system.nu01*system.sops.S1z+2*pi*system.nu02*system.sops.S2z;
    end
    % adds secular and pseudo-secular coupling to Zeeman Hamiltonian
    if isfield(system,'A') && isfield(system,'B') 
        hamA=2*pi*system.A*system.sops.S1z*system.sops.S2z;
        hamB=2*pi*system.B*system.sops.S1z*system.sops.S2x;
        system.ham=system.ham+hamA+hamB;
    else
        warning('No coupling constant given. Assuming default coupling of A =%5.2f GHz and B =%5.2f GHz.',default_A, default_B);
        system.A=default_A;
        system.B=default_B;
        hamA=2*pi*system.A*system.sops.S1z*system.sops.S2z;
        hamB=2*pi*system.B*system.sops.S1z*system.sops.S2x;
        system.ham=system.ham+hamA+hamB;
    end
end

if  ~isfield(system,'init_state') || isempty(system.init_state) 
    % if no initial state is given, one is created. Default is Mz.
    
    fprintf('Setup.m called without initial state, assuming high temperature limit.\n');
    if system.spins==1
        state=-system.sops.Sz;
    elseif system.spins==2
        state=-(system.sops.S1z+system.sops.S2z);
    end
    
else % if initial state is available, SPIDYAN checks the size of it.
    
    [m,n]=size(system.init_state);
    
    if m==n
        state=system.init_state;
    elseif (m==1 && n==3) || (m==3 && n==1)  
        % creates spin density matrix from magnetization vector         
        if system.spins==1
            state=-system.init_state(3)*system.sops.Sz-system.init_state(1)*system.sops.Sx-system.init_state(2)*system.sops.Sy;
        elseif system.spins==2
            state=-system.init_state(3)*(system.sops.S1z+system.sops.S2z)-system.init_state(1)*(system.sops.S1x+system.sops.S2x)-system.init_state(2)*(system.sops.S1y+system.sops.S2y);
        else
            error('Form of initial state not recognized by setup.m.');
        end
    end
    
end

if ~isfield(system,'eq') || isempty(system.eq) 
    % checks for equilibrium state and creates one if necessary
    if system.spins==1
        system.eq=-system.sops.Sz;
    elseif system.spins==2
        system.eq=-system.sops.S1z-system.sops.S2z;
    end
end
    
%checks the input for sufficiency and assumes default options if necessary
if ~exist('options','var') || ~isfield(options,'relaxation') || isempty(options.relaxation)
    options.relaxation=0;
    options.propagator=1;
    fprintf('Options.relaxation not defined, switching relaxation off.\n');
elseif options.relaxation==1 && ~isfield(options,'propagator')
    options.propagator=0;
    fprintf('No information on type of relaxation given. Calculating in Liouville space.\n');
elseif options.relaxation==0 && (~isfield(options,'propagator') || isempty(options.propagator) || options.propagator==0)
    options.propagator=1;
end


if ~isfield(system,'gamma')
    % creates relaxation superoperator from relaxation times
    if isfield(system,'T1') && isfield(system,'T2')
        
        if options.relaxation==1 && options.propagator==1
            
            if system.spins==1 && isfield(system,'T1') && isfield(system,'T2') && system.sqn1==1/2 && length(system.T1)==1 && length(system.T2)==1
                % make gamma AND relaxation matrix in Hilbert space for a
                % single spin from scalar input
                if system.T1==0
                    system.T1=system.default_T1;
                end
                if system.T2==0
                    system.T2=system.default_T2;
                end
                gamma=zeros(4,4);
                gamma(1,4)=-1/system.T1;
                gamma(4,1)=-1/system.T1;
                gamma(3,3)=1/system.T2;
                gamma(2,2)=1/system.T2;
                system.gamma=gamma;
                system.R = [1/system.T1 1/system.T2; 1/system.T2  1/system.T1];
                
            elseif system.spins==1 && isfield(system,'T1') && isfield(system,'T2') && system.sqn1==1/2 && ismatrix(system.T1) && ismatrix(system.T2)
                % make gamma AND relaxation matrix in Hilbert space for a
                % single spin from matrix input.
                system.T2(system.T2==0)=system.default_T2;
                system.T1(system.T1==0)=system.default_T1;
                gamma=zeros(4,4);
                gamma(1,4)=-1/system.T1(1,2);
                gamma(4,1)=-1/system.T1(1,2);
                gamma(3,3)=1/system.T2(1,2);
                gamma(2,2)=1/system.T2(1,2);
                system.gamma=gamma;
                system.R = [1/system.T1(1,2) 1/system.T2(1,2); 1/system.T2(1,2)  1/system.T1(1,2)];
                
            else
                error('Input is not sufficient to calculate relaxation matrix in Hilbert-space or system is not a twolevel system. Try switching off the propagator option or do not write relaxation times in matrix form.');
            end
        else
            system.gamma=changeup(system);
        end
    else
       system.gamma=[]; 
    end
end


if ~isfield(options,'det_op') || isempty(options.det_op)
    % sets detection operators to empty if not available
    fprintf('No detection operator given.\n');
    options.detect=[];
else
    %creates detection operators from strings
    options.detect=doubleplay(options.det_op,system);
end

% if options.truncation is true, SPIDYAN searches for transitions outside
% the Nyqist range, and sets the corresponding elements in the excitation
% operator to zero.
if isfield(options,'truncation') && ~isempty(options.truncation)
    low_cut = find(diff(diag(system.ham))/2/pi>0,1,'first');
    high_cut = find(diff(diag(system.ham))/2/pi<-options.awg.s_rate/2,1,'last');
    
    if (low_cut < high_cut)
        bla = high_cut;
        high_cut = low_cut;
        low_cut = bla;
    end
    
    if ~isempty(low_cut)
        system.sops.Sx(low_cut:end,low_cut:end) = zeros((2*system.sqn1+1)-low_cut+1);
    end
    if ~isempty(high_cut)
        system.sops.Sx(1:high_cut+1,1:high_cut+1) = zeros(high_cut+1);
    end
    
end

end
