function [experiment,options] = triple(sequence, options)
% [experiment,options] = triple(sequence, options)
% 
% The function triple uses the structure sequence to create a pulse
% experiment that can be used with the function homerun. The 
% sweep rate of the pulse is linear, unless SPIDYAN is asked
% to compensate for the resonator profile. In that case either the
% resonator parameters or a custom resonator profile has to be given.
% If options.resonator.active is true, the resonator profile is applied to
% the calculated pulse. The pulses before convolution are stored. The
% resonator profile is also stored in the structure experiment.
% 
% Input:
% sequence   structure with parameters for the pulse sequence
% .tp        Vector which contains the length of the pulses,
%            pulse delays and detection time. If set to zero for
%            a pulse with given flip angle and irradiation
%            strength, pulse length will be calculated.
% .beta      Vector with the flipangle of the pulses, must not
%            be given if pulse length and irradiation strength
%            are available. 
% .nu1       Irradiation strength of a pulse, must not given if
%            beta and tp are given.
% .frq       Cell or vector containing the excitation bandwidth.
%            If the excitation bandwidth stays constant through
%            the whole experiment it is possible to input it as a
%            vector. Else the cell format is required. For an example
%            see below. The excitation bandwidth has to be given
%            as an vector, with the initial chirp frequency as
%            first element and the final frequency as second
%            element.
% .t_rise    Scalar or vector, containing the rise time for the
%            pulse(s).
% .phase     scalar or vector, phase of the pulse, default is 0
% 
% options      structure containing additional parameters
% .det_op       cell containing the detection operators as string (S1z,S2z,
%               Sz,...) or in matrix matrix form. For more details see
%               documentation, can be empty.
% .detect       cell containing the detection operators in matrix form, if
%               input det was empty or not given, detect is empty.
% .no_detection cell, containing the event, that to exclude from detection,
%               eg.:
%                       options.no_detection={1, 3}
%               No detection during the 1st and 3rd event of the
%               experiment.
% .relaxation   0 or 1. if true, calculation includes relaxation. Uses
%               Liouville space. default is 0.
% .propagator   0 or 1. if true, the system is propagated with Hilbert space
%               with a propagator. Can not be true for systems with more 
%               than two levels AND relaxation on.
% .LO           frequency of the local oscillator [GHz], required for
%               computations in the lab frame.
% .labframe     0 or 1, default is 0. If true SPIDYAN simulates the
%               experiment in a local frame rotating at
%               options.LO+system.nu01.
% .down_conversion    0 or 1, if true, the detected signal will be 
%               transformed into the lab frame. Default is on.
% .no_dc        cell which containing the signals not to be down converted as
%               string or index of detection operator, optional.
% .cutoff_freq  cutoff frequency of the applied filter for down conversion
%               in MHz. Usually 200 MHz is a good value to start with. 
% .ui           0 or 1, optional, default is false. A simple
%               user-interface, which can be used to input relaxation
%               times.
% .truncation   if true, SPIDYAN checks for transitions outside the Nyquist
%               band and sets corresponding elements in the excitation
%               operator to zero. Only possible for single spin system.
% .awg          structure with information on awg, optional
%               if not given default values will be applied.
%   .awg.s_rate     sampling rate [GS/s], default is 12 GS/s
%   .awg.vert_res   vertical resolution [bit], default is 10 bit
%   .awg.phase      phase shift in rad, optional, default is 0
% 
% .display_filter   0 or 1, optional, default is false. If true displays the
%                   frequency response of the filter used for downconversion.
% .filtertruncation optional, strength of the filter for function 
%                   resonator, default is 1000. 
% .direct_evolution     vector, contains the indices of events which to
%                       propagate directly. can be used for interpulse
%                       delays to speed up calculation.
% .complex_excitation   0 or 1, optional, default is false. If true, complex
%                       microwave pulses are computed, which can be used for a
%                       complex excitation.
% .resonator     structure containing the resonator settings.
%   .resonator.comp_bw  if true, SPIDYAN compensates the pulse for the
%                       resonator
%   .resonator.Ql       quality factor of the resonator.
%   .resonator.f0       resonance frequency of the resonator.
%   .resonator.s_rate   sampling rate of the resonator profile.
%   .resonator.range    optional, frequency axis [GHz] for custom resonator
%                       profiles, if not given, the resonator profile
%                       is calculated from quality Ql, f0 and s_rate.
%   .resonator.nu       optional, resonator profile of custom resonator.
%   .resonator.active   vector, containing indices of events. Resonator
%                       profile is applied to pulses found in .active, e.g.
%                       [1 3] will change the first and third event only.
% 
% Output:
% experiment
% .pulse      structure which contains all information on the
%            pulses, empty element for free evolution.
%   .pulse{k}   structure with the k-th pulse
%   .pulse{k}.tp          total wave time (multiple of .dt) [ns]
%   .pulse{k}.dt          time step [ns]
%   .pulse{k}.t           time axis of wave
%   .pulse{k}.vert_res    vertical resulotion of wave.binary
%   .pulse{k}.arbitrary   waveform as double numbers, normalized to range 
%                         +/-1
%   .pulse{k}.binary      waveform in integer format 0...2^vert_res - 1
%   .pulse{k}.nu_init     Initial frequency of the sweep, if calculations
%                         are in the labframe, this will be frq(1)+LO
%   .pulse{k}.nu_final    Final frequency of the sweep, if calculations
%                         are in the labframe, this will be frq(2)+LO
%   .pulse{k}.nu1         pulse amplitude in MHz
%   .pulse{k}.beta        flipangle of pulse
%   .pulse{k}.frq         vector, excitation band
%   .pulse{k}.t_rise      rise time of the pulse
%   .pulse{k}.pcycle      phase cycling of the pulse, as defined in
%                         sequence.pcycle, 0 if no phase cycling
% .tp       vector containing the length of the pulses,
%           interpulse delays and detection time.
% .dt       time resolution (timesteps) of the experiment
% .i        index, used by homerun to process the experiment
% .o        offset, is used to calculate the time axis, will be
%           updated by homerun.
% .t        structure, contains the individual time axis of every event
% .taxis    vector, which contains time axis of the entire experiment
% 
% 
% Examples:
%  1)   Two pulses with an interpulse delay of 50 ns. The detection time is
%       500 ns. The flip angle of the first pulse is pi/2, the length 200
%       ns. The irradiation strength is calculated. The second pulse has a
%       length of 100 ns and a flip angle of pi. The excitation bandwidth
%       stays constant throughout the whole sequence, and can therefor be
%       given as either a cell with one vector or just a vector.
% 
%           sequence.tp=[200,50,100,500];
%           sequence.beta(1)=pi/2;
%           sequence.beta(3)=pi;
%           sequence.frq={[0.8 1.2]}; or sequence.frq=[0.8 1.2];
% 
%       Output:
%           experiment.pulse:   {[1x1 struct]  []  [1x1 struct]  []}
%           experiment.tp:      [200 50 100 500]
%           experiment.dt:       0.0833
%           experiment.i:       1
%           experiment.o:       0
% 
%  2)   Two pulses with an interpulse delay of 1000 ns. The bandwidth of
%       the second pulse is different from the first. No FID.
% 
%           sequence.tp=[2048 1000 2048];
%           sequence.nu1=[6.5 0 6.5];
%           sequence.frq={[1.5 .9] 0 [0.5 1.08]};
% 
%       Output:
%           experiment.pulse:   {[1x1 struct]  []  [1x1 struct]}
%           experiment.tp:      [2048 1000 2048]
%           experiment.dt:      0.0833
%           experiment.i:       1
%           experiment.o:       0
% 
% St. Pribitzer, 2014

% setting default parameters for AWG

defaultawg.s_rate=12;
defaultawg.vert_res=10;
defaultawg.channels=1;

default_t_rise=20;

% checks the input
if ~exist('options','var')
    options=[];
end

if ~isfield(sequence,'nu1')
    sequence.nu1=[];
end

if ~isfield(sequence,'beta')
    sequence.beta=[];
end

if isfield(options,'awg') && ~isempty(options.awg)
    
    awg=options.awg;
    a=0;
    % checks if all AWG fields are specified and assumes default values if
    % necessary
    if ~isfield(options.awg,'s_rate') ||  isempty(options.awg.s_rate) 
        awg.s_rate=defaultawg.s_rate;
        a=1;
    end
    
    if ~isfield(options.awg,'vert_res') ||  isempty(options.awg.vert_res) 
        awg.vert_res=defaultawg.vert_res;
        a=1;
    end
    
    if ~isfield(options.awg,'channels') ||  isempty(options.awg.channels) 
        awg.channels=defaultawg.channels;
    end
      
    if a
        % stores the new AWG if fields were missing
        options.awg=awg;
        fprintf('AWG was not fully defined, assumed default values for missing fields.\n');
    end
    
else
    % If no AWG is given, assumes default values for all fields
    awg=defaultawg;
    fprintf('No AWG given, assuming default values.\n');
    options.awg=awg;
end

if isfield(options,'resonator') && isfield(options.resonator, 'active') && ~isempty(options.resonator.active) && (~isscalar(options.resonator.active) || options.resonator.active~=0)
    % ensures complex wave form to be able to use resonator
    if ~isfield(options.awg,'channels') || options.awg.channels==1
        fprintf('Resonator requested, output of AWG is complex.\n');
    end
    options.awg.channels=2;
end

if isfield(options,'complex_excitation') && options.complex_excitation
    % sets number of channels to 2 for complex ecitation
    if ~isfield(options.awg,'channels') || options.awg.channels==1
        fprintf('Complex excitation requested, complex AWG output.\n');
    end
    options.awg.channels=2;
end

% creation of experiment
for k=1:length(sequence.tp)
    
    % checks if current event is a pulse or free evolution
    if ((isempty(sequence.nu1) || k>length(sequence.nu1) || ... 
            sequence.nu1(k)==0)) && ((isempty(sequence.beta) || ...
            k>length(sequence.beta) || sequence.beta(k)==0)) && ...
            (~isfield(sequence,'Q') || isempty(sequence.Q) || ...
            k>length(sequence.Q) || sequence.Q(k)==0)
            
        
        % for free evolution only the time is stored and the structure
        % pulse of the corresponding element is empty
        experiment.tp(k)=sequence.tp(k);
        experiment.pulse{k}=[];
        
        % creation of time step size if required
        if ~isfield(experiment,'dt') || isempty(experiment.dt)
            experiment.dt=1/options.awg.s_rate;
        end
        
        if exist('ringer','var') && ringer
            if experiment.tp(k)-offset<0
                warning('Inter-pulse delay or free evolution too short to compensate for ringing after pulse, skipping compensation. \n')
            else
                experiment.tp(k)=experiment.tp(k)-offset;
                ringer=0;
                fprintf('Free evolution length of event %1.0f has been changed from %4.2f to %4.2f ns due to ringing from the pulse. \n',k,sequence.tp(k),experiment.tp(k))
            end
            
        end
    
    else
        
        %  for a pulse SPIDYAN checks if it can find a rise time
        if isfield(sequence,'t_rise') && ~isempty(sequence.t_rise)
            if isscalar(sequence.t_rise)
                pulse.t_rise=sequence.t_rise;
                
            elseif k<=length(sequence.t_rise)
                pulse.t_rise=sequence.t_rise(k);
            
            else
                % if no rise time is given for the current event, SPIDYAN 
                % assumes the default rise time
                pulse.t_rise=default_t_rise;
                fprintf('Event %1.0f: No rise time given, assuming default rise time of %4.1f ns.\n', k, default_t_rise);
            
            end 
            
        else   
            % if no rise time is given, SPIDYAN assumes the default rise time
            pulse.t_rise=default_t_rise;
            fprintf('Event %1.0f: No rise time given, assuming default rise time of %4.1f ns.\n',k, default_t_rise);  
        
        end
        
        if isfield(sequence,'nu1') && ~isempty(sequence.nu1) && ...
                k<=length(sequence.nu1) && sequence.nu1(k)~=0
            % looks for pulse strength of current event
            pulse.nu1=sequence.nu1(k);
            
        else
            % if nu1 is not given or zero, SPIDYAN calculates the required
            % pulse strength from the flip angle
            pulse.nu1=[];
            
        end
        
        if isfield(sequence,'beta') && ~isempty(sequence.beta) && ...
                k<=length(sequence.beta) && sequence.beta(k)~=0
            % looks for flip angle of current pulse
            pulse.beta=sequence.beta(k);
            
        else 
            % if beta is not given or zero, SPIDYAN calculates the flip
            % angle from pulse length and strength
            pulse.beta=[];
            
        end
        
        if ~isempty(pulse.nu1) && ~isempty(pulse.beta)
            fprintf('Event %1.0f: Pulse strength and flip angle given, using pulse strength.\n',k);
        end
        
        if isfield(sequence,'tp') && ~isempty(sequence.tp) && ...
                k<=length(sequence.tp) && sequence.tp(k)~=0
            % gets pulse length
            pulse.tp=sequence.tp(k);
            
        else
            % if pulse length is not available or zero, it is calculated
            % from flip angle and pule strength
            pulse.tp=[];
            
        end
        
        
        if isfield(sequence,'frq') && ~isempty(sequence.frq) && ...
                iscell(sequence.frq)
            
            % tests if frq is the same for every pulse
            if length(sequence.frq)==1
                
                if length(sequence.frq{1})==1 % for hard pulses 
                    pulse.frq=[sequence.frq{1} sequence.frq{1}];
                else % for chirps
                    pulse.frq=sequence.frq{1};
                end
                
            else   
            % or different for each event 
            
                if length(sequence.frq{k})==1 % for hard pulses 
                    pulse.frq=[sequence.frq{k} sequence.frq{k}];
                else % for chirps
                    pulse.frq=sequence.frq{k};
                end
            end        
            
        elseif isfield(sequence,'frq') && ~isempty(sequence.frq) && ...
                isvector(sequence.frq)
            
            % creates the excitation band in case sequence.frq is given as a
            % vector
            if length(sequence.frq)==1
                pulse.frq=[sequence.frq sequence.frq];
            else
                pulse.frq=sequence.frq;
            end
            
        else
            error('Error: No information on excitation bandwidth given.');
        
        end
        
        if isfield(sequence,'Q') && ~isempty(sequence.Q)
            
            if k<=length(sequence.Q)
                pulse.Q=sequence.Q(k);
            else
                pulse.Q=[];
            end
            
        else
            pulse.Q=[];
        end
        
        
        
        if isfield(sequence,'phase') && ~isempty(sequence.phase)
            
            if length(sequence.phase)==1      
                pulse.phase=sequence.phase;
            elseif k<=length(sequence.phase)
                pulse.phase=sequence.phase(k);
            else
                pulse.phase=0;
            end
                        
        else
            pulse.phase=0;
         
        end
        
        % number of current event, required for debugging and feedback to 
        % user
        pulse.eventno=k; 
        
        % creates pulse, missing pulse parameters are calculated within
        % hitbypitch
        
        if isfield(sequence,'pcycle') && k<=length(sequence.pcycle) && ~isempty(sequence.pcycle{k}) 
            
            pulse.phase=pulse.phase+sequence.pcycle{k}(1,1);
                
            [chirp, options]=hitbypitch(pulse, options);
            
            for kk=2:size(sequence.pcycle{k},1)
                pulse.phase=pulse.phase+sequence.pcycle{k}(kk,1);
                
                pulse_pcyc=hitbypitch(pulse, options);
                     
                chirp.binary(kk,:)= pulse_pcyc.binary;
                chirp.arbitrary(kk,:)= pulse_pcyc.arbitrary;
                
            end
            
            chirp.pcycle=sequence.pcycle{k}; 
        else
            [chirp, options]=hitbypitch(pulse, options);
            chirp.pcycle=0;
        end
         
        % convolution of pulse with resonator profile if requested
        if isfield(options,'resonator') && isfield(options.resonator,'active') && any(options.resonator.active==k)
                if exist('ringing','var') && ringer
                    warning('Two pulses without delay or interpulse not sufficient to compensate for ringing. Ringing arises from convultion of the resonator profile with the original chirp. Event lengths will no longer fit and are shifted.\n')
                end
                chirp=resonator(chirp,options);
                ringer=1;
                offset=chirp.tp-chirp.tp_orig;
                if ~isfield(sequence,'tp_orig')
                    experiment.tp_orig=sequence.tp;
                end
        end
        
        if isfield(sequence,'excite') && ~isempty(sequence.excite)
            
            if ~iscell(sequence.excite)
                excite=sequence.excite;
            elseif length(sequence.excite)==1  
                excite=sequence.excite{1};
            elseif k<=length(sequence.excite)
                excite=sequence.excite{k};
            else
                excite=1;
            end
            
            if size(excite,1)==size(excite,2) && size(excite,1) ~= 1
                chirp.xop=excite;
            end           
        else
            excite=[];
        end
        
        chirp.excite=excite;
        
        % stores the pulse into experiment
        experiment.pulse{k}=chirp;
        experiment.tp(k)=experiment.pulse{k}.tp;
        experiment.dt=experiment.pulse{k}.dt;     

    end
end

% creates indices for homerun
experiment.i=1; % counter for event position
experiment.o=0; % offset of timeaxis, updated after every event in homerun

% if resonator is available it is stored to experiment as well
if isfield(options,'resonator') && isfield(options.resonator,'active') && ~isempty(options.resonator.active) && (~isscalar(options.resonator.active) || options.resonator.active~=0)
    experiment.resonator=options.resonator;
end

if isfield(sequence,'pcycle')
    experiment.pcycle=sequence.pcycle;
end

end

