function [state, detected_signal, experiment, tables]=wave_propagation(state, system, experiment, options, tables)
% [state, detected_signal, experiment, tables] = wave_propagation(state, system, experiment, options, tables)
% 
% propagates the state of any system under irradation or no
% irradiation by calculation in the Hilbert space or in Liouville space.
% If relaxation is switched off, propagation will be calculated in the
% Hilbert space exclusively.
%
% Input:
% state        initial state, density matrix
% 
% system        structure with the spin system
% .nu01         resonance frequency of spin 1 in GHz, optional only if
%               system.ham is given
% .nu02         resonance frequency of spin 2 in GHz, optional
% .ham          hamilton operator the system, can be given or
%               is created by setup.m
% .sqn1         nuclear spin of the first spin, is assumed as 1/2 if no 
%               value is given
% .sqn2         nuclear spin of the second spin, optional
% .init_state   initial state, can be a magnetization vector, density matrix or
%               empty. if empty thermal equilibrium will be assumed. Example 
%               for a vector system.init_state=[0 0 1].
% .spins        number of spin, optional, if 2 and no sqn2 (empty or not
%               defined), sqn2 is assumed to be 1/2.
% .A,.B         coupling constants, only needed for a two spin system
% .D            symmetric zero-field-splitting parameter in GHz
% .E            asymmetric zero-field-splitting parameter in GHz
% .theta        first angle in rad, for calculation of ZFS hamiltonian
% .phi          second angle in rad, for calculation of ZFS hamiltonian
% .omega1       zero-field-splitting frequency, calculated from system.D 
%               with fastball.
% .omega21,     zero-field-splitting frequencies for higher order
% .omega22      Hamiltonian terms                        
% .T1           double or matrix with longitudinal relaxation times, 
%               detailed instructions see below.
% .T2           vector with transversal relaxation times, detailed
%               instructions see below.
% .gamma        relaxation matrix, optional, else calculated from T1 and T2
% .default_T1   a default value for T1 relaxation, optional. Is applied to 
%               all non-defined T1 pathways.
% .default_T2   a default value for T2 relaxation, optional. Is applied to 
%               all non-defined T2 pathways
% .eq           equilibrium state, density matrix, optional, created as
%               thermal equilibrium if not else specified
% .R            relaxation matrix in Hilbert space, only available for
%               system with one spin and S=1/2
% .sops         structure containing the spin operators
% 
% experiment
% .pulse      structure which contains all information on the
%            pulses, empty element for free evolution.
%   .pulse{k}   structure with the k-th pulse
%   .pulse{k}.tp          total wave time (multiple of .dt) [ns]
%   .pulse{k}.dt          time step [ns]
%   .pulse{k}.t           time axis of wave
%   .pulse{k}.vert_res    vertical resulotion of wave.binary
%   .pulse{k}.arbitrary   waveform as double numbers, normalized to range 
%                         +/-1
%   .pulse{k}.binary      waveform in integer format 0...2^vert_res - 1
%   .pulse{k}.nu_init     Initial frequency of the sweep, if calculations
%                         are in the labframe, this will be frq(1)+LO
%   .pulse{k}.nu_final    Final frequency of the sweep, if calculations
%                         are in the labframe, this will be frq(2)+LO
%   .pulse{k}.nu1         pulse amplitude in MHz
%   .pulse{k}.beta        flipangle of pulse
%   .pulse{k}.frq         vector, excitation band
%   .pulse{k}.t_rise      rise time of the pulse
%   .pulse{k}.pcycle      phase cycling of the pulse, as defined in
%                         sequence.pcycle, 0 if no phase cycling
% .tp       vector containing the length of the pulses,
%           interpulse delays and detection time.
% .dt       time resolution (timesteps) of the experiment
% .i        index, used by homerun to process the experiment
% .o        offset, is used to calculate the time axis, will be
%           updated by homerun.
% .t        structure, contains the individual time axis of every event
% .taxis    vector, which contains time axis of the entire experiment
% 
% options      structure containing additional parameters
% .det_op       cell containing the detection operators as string (S1z,S2z,
%               Sz,...) or in matrix matrix form. For more details see
%               documentation, can be empty.
% .detect       cell containing the detection operators in matrix form, if
%               input det was empty or not given, detect is empty.
% .no_detection cell, containing the event, that to exclude from detection,
%               eg.:
%                       options.no_detection={1, 3}
%               No detection during the 1st and 3rd event of the
%               experiment.
% .relaxation   0 or 1. if true, calculation includes relaxation. Uses
%               Liouville space. default is 0.
% .propagator   0 or 1. if true, the system is propagated with Hilbert space
%               with a propagator. Can not be true for systems with more 
%               than two levels AND relaxation on.
% .LO           frequency of the local oscillator [GHz], required for
%               computations in the lab frame.
% .labframe     0 or 1, default is 0. If true SPIDYAN simulates the
%               experiment in a local frame rotating at
%               options.LO+system.nu01.
% .down_conversion    0 or 1, if true, the detected signal will be 
%               transformed into the lab frame. Default is on.
% .no_filt      cell which containing the signals not to be filtered as
%               string or index of detection operator, optional.
% .cutoff_freq  cutoff frequency of the applied filter for down conversion
%               in MHz. Usually 200 MHz is a good value to start with. 
% .ui           0 or 1, optional, default is false. A simple
%               user-interface, which can be used to input relaxation
%               times.
% .truncation   if true, SPIDYAN checks for transitions outside the Nyquist
%               band and sets corresponding elements in the excitation
%               operator to zero. Only possible for single spin system.
% .awg          structure with information on awg, optional
%               if not given default values will be applied.
%   .awg.s_rate     sampling rate [GS/s], default is 12 GS/s
%   .awg.vert_res   vertical resolution [bit], default is 10 bit
%   .awg.phase      phase shift in rad, optional, default is 0
% 
% .display_filter   0 or 1, optional, default is false. If true displays the
%                   frequency response of the filter used for downconversion.
% .filtertruncation optional, strength of the filter for function 
%                   resonator, default is 1000. 
% .direct_evolution     vector, contains the indices of events which to
%                       propagate directly. can be used for interpulse
%                       delays to speed up calculation.
% .complex_excitation   0 or 1, optional, default is false. If true, complex
%                       microwave pulses are computed, which can be used for a
%                       complex excitation.
% .resonator     structure containing the resonator settings.
%   .resonator.comp_bw  if true, SPIDYAN compensates the pulse for the
%                       resonator
%   .resonator.Ql       quality factor of the resonator.
%   .resonator.f0       resonance frequency of the resonator.
%   .resonator.s_rate   sampling rate of the resonator profile.
%   .resonator.range    optional, frequency axis [GHz] for custom resonator
%                       profiles, if not given, the resonator profile
%                       is calculated from quality Ql, f0 and s_rate.
%   .resonator.nu       optional, resonator profile of custom resonator.
%   .resonator.active   vector, containing indices of events. Resonator
%                       profile is applied to pulses found in .active, e.g.
%                       [1 3] will change the first and third event only.
% 
% tables        structure containing lookup tables
% .sigss        optional, table with steadystate density operators. if 
%               available, accelerates calulcation a lot. can only be used,
%               for pulses with the same nu1.
% .L            optional, table with liouvillians.
%               if given, will accelerate calulcation a lot. 
%               can only be used, if the exact same pulse was 
%               already used on the same system in a previous 
%               experiment.
% .nu1          irradtiation strength of the pulse for which
%               the tables are given. If not the same as the
%               applied pulse, tables will be calculated new.
% 
% Output:
% state                 final state, output in the same form as state
%                       input.
% 
% detected_signal       detected signal in the rotating frame, empty if
%                       no detection operator is given.
% 
% experiment             
%           .pulse      structure which contains all information on the
%                       waveform. See mk_chirp for more details.
%           .tp         vector containing the length of the pulses,
%                       interpulse delays and detection time.
%           .dt         time resolution (timesteps) of the experiment
%           .i          index, used by homerun to process the experiment
%           .o          updated offset, equals total time of the experiment
%                       after processing by wave_propagation.
%           .t          cell, which contains the timeaxis of the
%                       respective pulse or inter pulse delay.
% 
% tables        structure containing lookup tables
% .sigss        optional, table with steadystate density operators. if 
%               available, accelerates calulcation a lot. can only be used,
%               for pulses with the same nu1.
% .L            optional, table with liouvillians.
%               if given, will accelerate calulcation a lot. 
%               can only be used, if the exact same pulse was 
%               already used on the same system in a previous 
%               experiment.
% .nu1          irradtiation strength of the pulse for which
%               the tables are given. If not the same as the
%               applied pulse, tables will be calculated new.
%
% If input data is wrong or inconsistent, a warning is issued to the
% command window.
%
% St. Pribitzer, 2014


%checks the input for sufficiency and assumes default options if necessary
if ~exist('options','var') || ~isfield(options,'relaxation') || isempty(options.relaxation)
    options.relaxation=0;
    options.propagator=1;
    fprintf('Options.relaxation not defined, switching relaxation off.\n');
elseif options.relaxation==1 && ~isfield(options,'propagator')
    options.propagator=0;
    fprintf('No information on type of relaxation given. Calculating in Liouville space.\n');
elseif options.relaxation==0 && (~isfield(options,'propagator') || isempty(options.propagator) || options.propagator==0)
    options.propagator=1;
end


if ~isempty(experiment.pulse{experiment.i}) % this checks if the current element in the sequence is a pulse  
    
    wave=experiment.pulse{experiment.i};
    waves.arbitrary=wave.arbitrary;
    waves.binary=wave.binary;
      
    state0=state;
    
    if size(wave.pcycle,1)>1
        
        loopstate=zeros(size(state));
        norm=sumabs(wave.pcycle(:,2));
      
    end
    
    
    for rr=1:size(wave.pcycle,1)

        state=state0;
        
        if isfield(options,'complex_excitation') && options.complex_excitation
            wave.arbitrary_imag=imag(wave.arbitrary);
            wave.binary_imag=imag(wave.binary);
            complex=1;
        else
            complex=0;
        end
        
        wave.arbitrary=real(waves.arbitrary(rr,:));
        wave.binary=real(waves.binary(rr,:));
        
        MHz2GHz=1e-3;
        
        % check for system input completeness
        if ~isfield(system,'ham'),
            error('Warning: wave_propagation called without Hamiltonian.');
        end
        
        if isfield(wave,'arbitrary') && ~isfield(wave,'dt') && isfield(wave,'tp'),
            wave.dt=wave.tp/length(wave.arbitrary);
        end
        
        n=ceil(wave.tp/wave.dt);
        t=linspace(0,n*wave.dt,n+1);
        
        if ~isfield(wave,'nu1'),
            error('Error: wave_propagation called without nu1.');
        end
        
        % proper amplitude scaling
        wave.arbitrary=wave.arbitrary*2*2*pi*wave.nu1*MHz2GHz; % factor 2 required because of linearly polarized irradiation
        
        % moves the wave in binary format to -vert_res/2...vert_res/2-1 for proper
        % scaling
        vert_res=2^wave.vert_res;
        binary=wave.binary-vert_res/2;
        scale=max(wave.arbitrary)/max(binary);
        
        Utable=cell(1,vert_res);
        
        sig=state;
        ham0=system.ham;
        
        % create excitation operator
        if isfield(wave,'xop') && ~isempty(wave.xop)
            xop=wave.xop;
        elseif isempty(wave.excite)
            if system.spins==1
                xop=system.sops.Sx;
            elseif system.spins==2
                xop=system.sops.S1x;
            end
        elseif system.spins==2
            if length(wave.excite)==2
                if isequal([1 1],wave.excite) 
                    xop=system.sops.S1x+system.sops.S2x;
                elseif isequal([1 0],wave.excite)
                    xop=system.sops.S1x;
                elseif isequal([0 1],wave.excite)
                    xop=system.sops.S2x;
                else
                    error('excitation operator not recognized')
                end
            elseif wave.excite==1
                xop=system.sops.S1x;
            else
                error('Excitation operator not recognized')
            end
        elseif system.spins==1
            xop=system.sops.Sx;
        end
        
        if options.propagator==1 && exist('tables','var') && ~isempty(tables) && tables.nu1==wave.nu1 && isequal(tables.ham,system.ham) % calculates and stores the propagators if required
        
            Utable=tables.U;
        
        elseif options.propagator==1
            
            if system.spins==2
                
                if size(wave.pcycle,1)>1
                    for k=0:vert_res-1,
                        ham1=scale*(k-vert_res/2)*xop;
                        ham=ham0+ham1;
                        U=expm(-1i*ham*wave.dt);
                        Utable{k+1}=U;
                    end
                    
                else
                    for k=min(wave.binary):max(wave.binary),
                        %                 ham1=scale*(k-vert_res/2)*(system.sops.S1x+system.sops.S2x);
                        ham1=scale*(k-vert_res/2)*xop;
                        ham=ham0+ham1;
                        U=expm(-1i*ham*wave.dt);
                        Utable{k+1}=U;
                    end
                end
                                         
            elseif system.spins==1
                
                if size(wave.pcycle,1)>1
                    for k=0:vert_res-1,
                        %                 ham1=scale*(k-vert_res/2)*system.sops.Sx;
                        ham1=scale*(k-vert_res/2)*xop;
                        ham=ham0+ham1;
                        U=expm(-1i*ham*wave.dt);
                        Utable{k+1}=U;
                    end
                else
                    for k=min(wave.binary):max(wave.binary),
                        %                 ham1=scale*(k-vert_res/2)*system.sops.Sx;
                        ham1=scale*(k-vert_res/2)*xop;
                        ham=ham0+ham1;
                        U=expm(-1i*ham*wave.dt);
                        Utable{k+1}=U;
                    end
                end

            end
            
            tables.nu1=wave.nu1;
            tables.U=Utable;
            tables.ham=system.ham;
            
        end
        
        if options.relaxation==0   % propagates without relaxation
            
            if isfield(options,'detect') && ~isempty(options.detect)
                
                ld=length(options.detect);
                detected_signal=zeros(ld,length(wave.arbitrary));
                norms=zeros(1,length(options.detect));
                
                for kk=1:length(options.detect),
                    norms(kk) = trace(options.detect{kk}*options.detect{kk}');
                    detected_signal(kk,1) = trace(options.detect{kk}*sig)/norms(kk);
                end
                
                for k=1:length(wave.arbitrary)
                    
                    if ~complex
                        U=Utable{wave.binary(k)+1};
                        sig=U*sig*U';
                    else
                        
                        if system.spins==2
                            ham1=scale/2*(wave.binary(k)-vert_res/2)*(system.sops.S1x)+scale/2*(wave.binary_imag(k)-vert_res/2)*(system.sops.S1y);
                        else
                            ham1=scale/2*(wave.binary(k)-vert_res/2)*(system.sops.Sx)+scale/2*(wave.binary_imag(k)-vert_res/2)*(system.sops.Sy);
                        end
                        
                        ham=ham0+ham1;
                        U=expm(-1i*ham*wave.dt);
                        sig=U*sig*U';
                        
                    end
                    
                    for j=1:ld
                        detected_signal(j,k+1)=trace(options.detect{j}*sig)/norms(j);
                    end
                end
            else
                
                for k=1:length(wave.arbitrary)
                    
                    if ~complex
                        U=Utable{wave.binary(k)+1};
                        sig=U*sig*U';
                    else
                        
                        if system.spins==2
                            ham1=scale/2*(wave.binary(k)-vert_res/2)*(system.sops.S1x)+scale/2*(wave.binary_imag(k)-vert_res/2)*(system.sops.S1y);
                        else
                            ham1=scale/2*(wave.binary(k)-vert_res/2)*(system.sops.Sx)+scale/2*(wave.binary_imag(k)-vert_res/2)*(system.sops.Sy);
                        end
                        
                        ham=ham0+ham1;
                        U=expm(-1i*ham*wave.dt);
                        sig=U*sig*U';
                        
                    end
                end
                
                detected_signal=[];
                
            end
            
            state=sig;
            
           
        elseif options.relaxation==1 && options.propagator==1                   % relaxation with propagator in hilbert space
            
            R=system.R*-wave.dt; % relaxation matrix
            sigeq=system.eq;
            
            if isfield(options,'detect') && ~isempty(options.detect) % propagates and detects
                
                ld=length(options.detect);
                detected_signal=zeros(ld,length(wave.arbitrary));
                norms=zeros(1,length(options.detect));
                
                for kk=1:length(options.detect),
                    norms(kk) = trace(options.detect{kk}*options.detect{kk}');
                    detected_signal(kk,1) = trace(options.detect{kk}*sig)/norms(kk);
                end
                
                for k=1:length(wave.arbitrary)
                    U=Utable{wave.binary(k)+1};
                    sig=sig+(sig-sigeq).*R/2;
                    sig=U*sig*U';
                    sig=sig+(sig-sigeq).*R/2;
                    for j=1:ld
                        detected_signal(j,k+1)=trace(options.detect{j}*sig)/norms(j);
                    end
                end
                
            else        %only propagation, no detection, final state is returned
                
                for k=1:length(wave.arbitrary)
                    U=Utable{wave.binary(k)+1};
                    sig=sig+(sig-sigeq).*R/2;
                    sig=U*sig*U';
                    sig=sig+(sig-sigeq).*R/2;
                end
                
                detected_signal=[];
            end
            
            state=sig;
            
            tables=[];
            
        elseif options.relaxation==1 && options.propagator==0    % progation with relaxation in Liouville space
            
            [n,~]=size(sig);
            sigvec=reshape(sig,n*n,1); %reshaping the density matrix
            sig_eq=reshape(system.eq,n*n,1);
            
            if exist('tables','var') && ~isempty(tables) && tables.nu1==wave.nu1 && isequal(tables.ham,system.ham) %check for available tables
                
                Ltable=tables.L;
                sigsstable=tables.sigss;
                
            elseif system.spins==2      % setup tables in case they dont exist or are incorrect
                
                sigsstable=cell(1,vert_res);
                Ltable=cell(1,vert_res);
                
                for k=min(wave.binary):max(wave.binary),
                    %                 ham1=scale*(k-vert_res/2)*(system.sops.S1x+system.sops.S2x);
                    ham1=scale*(k-vert_res/2)*xop;
                    ham=ham0+ham1;
                    hamm=kron(eye(n,n),ham)-kron(ham,eye(n,n));
                    L=-1i*hamm-system.gamma;
                    sigss=system.gamma*sig_eq;      % steady state solutions for the denisty matrices
                    sigss=-L\sigss;
                    sigsstable{k+1}=sigss;
                    L=expm(L*wave.dt);              %calculations of the Liouvillians
                    Ltable{k+1}=L;
                end
                
            elseif system.spins==1
                
                sigsstable=cell(1,vert_res);
                Ltable=cell(1,vert_res);
                
                for k=min(wave.binary):max(wave.binary),
                    %                 ham1=scale*(k-vert_res/2)*system.sops.Sx;
                    ham1=scale*(k-vert_res/2)*xop;
                    ham=ham0+ham1;
                    hamm=kron(eye(n,n),ham)-kron(ham,eye(n,n));
                    L=-1i*hamm-system.gamma;
                    sigss=system.gamma*sig_eq;
                    sigss=-L\sigss;
                    sigsstable{k+1}=sigss;
                    L=expm(L*wave.dt);
                    Ltable{k+1}=L;
                end
                
            end
            
            if isfield(options,'detect') && ~isempty(options.detect) % propagates the spin sytem with a time trace
                
                ld=length(options.detect);
                detected_signal=zeros(ld,length(wave.arbitrary));
                norms=zeros(1,length(options.detect));
                
                for kk=1:length(options.detect),
                    norms(kk) = trace(options.detect{kk}*options.detect{kk}');
                    detected_signal(kk,1) = trace(options.detect{kk}*sig)/norms(kk);
                end
                
                for k=1:length(wave.arbitrary)
                    L=Ltable{wave.binary(k)+1};
                    sigss=sigsstable{wave.binary(k)+1};
                    sigvec=sigss+L*(sigvec-sigss);
                    sig=reshape(sigvec,n,n);
                    for j=1:ld
                        detected_signal(j,k+1)=trace(options.detect{j}*sig)/norms(j);
                    end
                end
                
            else % propagates the spin sytem directly to its final state
                
                for k=1:length(wave.arbitrary)
                    L=Ltable{wave.binary(k)+1};
                    sigss=sigsstable{wave.binary(k)+1};
                    sigvec=sigss+L*(sigvec-sigss);
                end
                
                sig=reshape(sigvec,n,n);
                detected_signal=[];
                
            end
            
            state=sig;
            
            tables.L=Ltable;        % stores the tables
            tables.sigss=sigsstable;
            tables.nu1=wave.nu1;
            
        end
        
        if size(wave.pcycle,1)>1
            
            pcycle_det=wave.pcycle(rr,2)/norm;
            loopstate=loopstate+pcycle_det*sig;
                       
            if isfield(options,'detect') && ~isempty(options.detect)
                if exist('pcycle_sig','var')
                        pcycle_sig=pcycle_sig+pcycle_det*detected_signal;
                else
                    pcycle_sig=zeros(size(detected_signal));
                    
                    for lr=1:length(options.detect)
                        pcycle_sig=pcycle_det*detected_signal;
                    end
                end
                
            end

        end
        
    end
    
    if size(wave.pcycle,1)>1
        state=loopstate;
        if isfield(options,'detect') && ~isempty(options.detect)
            detected_signal = pcycle_sig;
        end
        
    end
    
    if exist('experiment','var') && ~isempty(experiment.o)
        experiment.t{experiment.i}=t+experiment.o;
        experiment.o=experiment.o+t(end);
        experiment.i=experiment.i+1;
    else
        experiment.t=t;
    end
              
else %this is executed if no pulse is found for the current element, free evolution of the spin system
    
    if options.relaxation==1 && options.propagator==1                        %evolution in hilbertspace with relaxation
       
        if system.spins==1 && system.sqn1==0.5
           
            if isfield(options,'direct_evolution') && ~isempty(options.direct_evolution) && any(experiment.i==options.direct_evolution)
                t(2)=experiment.tp(experiment.i);
                dt=t(2);
                
            else
                if ~isfield(experiment,'dt') || isempty(experiment.dt)  %if no timegrid is given, this creates a first guess from the length of the free evolution
                    experiment.dt=experiment.tp(experiment.i);
                end

                if 1/experiment.dt<2*system.nu01 %checks if the time grid is small enough to fullfill Nyquist
                    
                    t=linspace(0,experiment.tp(experiment.i),round(experiment.tp(experiment.i)*6*system.nu01)+1); %If Nyquist is not met, a new time grid is created from the resonance frequency of the spinsystem
                    dtn=t(2)-t(1);
                    warning('timesteps too big for evolution, new timesteps have been created that fit the nyquist criterium.\n old: %6.0f ns; new:  %6.3f ns \n',experiment.dt,dtn);
                    experiment.dt=dtn;
                    
                else
                    
                    t=linspace(0,experiment.tp(experiment.i),ceil(experiment.tp(experiment.i)/experiment.dt)+1);        
               
                end
                
                dt=experiment.dt;
                
            end
            
                sig=state; 
                ham=system.ham;
                U=expm(-1i*ham*dt);          %creates the progator from the system Hamiltonian
                sigeq=system.eq;
                R=system.R*-dt;
                
            if isfield(options,'detect') && ~isempty(options.detect) %free evolution with detection
                
                ld=length(options.detect);
                detected_signal=zeros(ld,length(t)); 
                norms=zeros(1,length(options.detect));
                
                for kk=1:length(options.detect), %calulates expectation values for the initial state for all detection operators
                    norms(kk) = trace(options.detect{kk}*options.detect{kk}');
                    detected_signal(kk,1) = trace(options.detect{kk}*sig)/norms(kk);
                end
                
                for k=2:length(t) %evolves system and calculates expectation values
                    
                    sig=sig+(sig-sigeq).*R/2;
                    sig=U*sig*U';
                    sig=sig+(sig-sigeq).*R/2;
                    
                    for j=1:ld
                        detected_signal(j,k)=trace(options.detect{j}*sig)/norms(j);
                    end
                    
                end
                
            else    %free evolution without detection.
                
                for k=2:length(t)
                    sig=sig+(sig-sigeq).*R/2;
                    sig=U*sig*U';
                    sig=sig+(sig-sigeq).*R/2;                  
                end
                
                detected_signal=[];
                
            end

            state=sig;

            if exist('experiment','var') && ~isempty(experiment.o) %handles the indexing and timeoffsets
                experiment.t{experiment.i}=t+experiment.o;
                experiment.o=experiment.o+t(end);
                experiment.i=experiment.i+1;
            else
                experiment.t=t;
            end

            tables=[];
       else
           error('wave_propagation called with relaxation in Hilbert, but either system contains two spins or S>1/2.')
       end

    elseif options.relaxation==1  %evolution in liouville space
        
        [m,~]=size(system.sops.E);
        
        if isfield(system,'eq'), % checks and reshapes equilibrium state
            
            sig_eq=system.eq;
            [o, p]=size(sig_eq);
            
            if o==p
                sig_eq=reshape(sig_eq,m*m,1);
            end      
            
        elseif system.spins==2,
            sig_eq=reshape(-system.sops.S1z-system.sops.S2z,m*m,1);
        elseif system.spins==1
            sig_eq=reshape(-system.sops.Sz,m*m,1);
        end

        if isfield(options,'direct_evolution') && ~isempty(options.direct_evolution) && any(experiment.i==options.direct_evolution)
            
            t(2)=experiment.tp(experiment.i);
            dt=t(2);
                
        else
            
            if ~isfield(experiment,'dt') || isempty(experiment.dt)  %if no timegrid is given, this creates a first guess from the length of the free evolution
                experiment.dt=experiment.tp(experiment.i);
            end

            if 1/experiment.dt<2*system.nu01 %checks if the time grid is small enough to fullfill Nyquist
                t=linspace(0,experiment.tp(experiment.i),round(experiment.tp(experiment.i)*6*system.nu01)+1); %If Nyquist is not met, a new time grid is created from the resonance frequency of the spinsystem
                dtn=t(2)-t(1);
                warning('timesteps too big for evolution, new timesteps have been created that fit the nyquist criterium.\n old: %6.0f ns; new:  %6.3f ns \n',experiment.dt,dtn);
                experiment.dt=dtn;
            else
                t=linspace(0,experiment.tp(experiment.i),ceil(experiment.tp(experiment.i)/experiment.dt)+1);        
            end
                dt=experiment.dt;
        end
        
        sig=state;
        [n,~]=size(sig);
        sigvec=reshape(sig,n*n,1);
        ham=system.ham;

        % create Liouvillian and steady state density matrix for free evolution
        ham=kron(eye(n,n),ham)-kron(ham,eye(n,n)); 
        L=-1i*ham-system.gamma;
        sigss=system.gamma*sig_eq;
        sigss=-L\sigss;
        L=expm(L*dt);
        
        if isfield(options,'detect') && ~isempty(options.detect) %evolves the spinsytem in Liouville space 
            
            ld=length(options.detect);
            detected_signal=zeros(ld,length(t)); 
            norms=zeros(1,length(options.detect));
            
            for kk=1:length(options.detect),
                norms(kk) = trace(options.detect{kk}*options.detect{kk}');
                detected_signal(kk,1) = trace(options.detect{kk}*sig)/norms(kk);
            end
            
            for k=2:length(t)
                sigvec=sigss+L*(sigvec-sigss);
                sig=reshape(sigvec,n,n);
                for j=1:ld
                    detected_signal(j,k)=trace(options.detect{j}*sig)/norms(j);
                end                
            end
            
        else
            
            for k=2:length(t)
                sigvec=sigss+L*(sigvec-sigss);
            end
            
            sig=reshape(sigvec,n,n);
            detected_signal=[];
            
        end

        state=sig;
 
        if exist('experiment','var') && ~isempty(experiment.o) %handles the indexing and timeoffsets
            experiment.t{experiment.i}=t+experiment.o;
            experiment.o=experiment.o+t(end);
            experiment.i=experiment.i+1;
        else
            experiment.t=t;
        end
        
        tables=[];
        
    else  %evolution with propagator method and without relaxation

        if isfield(options,'direct_evolution') && ~isempty(options.direct_evolution) && any(experiment.i==options.direct_evolution)
            t(2)=experiment.tp(experiment.i);
            dt=t(2);

        else
            if ~isfield(experiment,'dt') || isempty(experiment.dt)  %if no timegrid is given, this creates a first guess from the length of the free evolution
                experiment.dt=experiment.tp(experiment.i);
            end

            if 1/experiment.dt<2*system.nu01 %checks if the time grid is small enough to fullfill Nyquist
                t=linspace(0,experiment.tp(experiment.i),round(experiment.tp(experiment.i)*6*system.nu01)+1); %If Nyquist is not met, a new time grid is created from the resonance frequency of the spinsystem
                dtn=t(2)-t(1);
                warning('timesteps too big for evolution, new timesteps have been created that fit the nyquist criterium.\n old: %6.0f ns; new:  %6.3f ns \n',experiment.dt,dtn);
                experiment.dt=dtn;
            else
                t=linspace(0,experiment.tp(experiment.i),ceil(experiment.tp(experiment.i)/experiment.dt)+1);        
            end
            dt=experiment.dt;
        end

        sig=state; 

        ham=system.ham;
        U=expm(-1i*ham*dt);
        
        if isfield(options,'detect') && ~isempty(options.detect) %propagates with detection
            
            ld=length(options.detect);
            detected_signal=zeros(ld,length(t)); 
            norms=zeros(1,length(options.detect));
            
            for kk=1:length(options.detect),
                norms(kk) = trace(options.detect{kk}*options.detect{kk}');
                detected_signal(kk,1) = trace(options.detect{kk}*sig)/norms(kk);
            end
          
            for k=2:length(t)
                
                sig=U*sig*U';
                
                for j=1:ld
                    detected_signal(j,k)=trace(options.detect{j}*sig)/norms(j);
                end
            end
            
        else %propagates without detection
            
            for k=2:length(t)
                 sig=U*sig*U';
            end
            
            detected_signal=[];
        end

        state=sig;

        if isfield(experiment,'o') && ~isempty(experiment.o) %handles the indexing and timeoffsets
            experiment.t{experiment.i}=t+experiment.o;
            experiment.o=experiment.o+t(end);
            experiment.i=experiment.i+1;
        else
            experiment.t=t;
        end
        tables=[];
    end
end

end


